@@ -1,4 +1,4 @@
-/* $Id: color.c,v 1.38 2006/01/02 17:09:35 dolorous Exp $ */
+/* $Id: color.c,v 1.39 2006/05/28 18:51:48 dolorous Exp $ */
 /**************************************************************************
  *   color.c                                                              *
  *                                                                        *
@@ -142,7 +142,7 @@ void color_update(void)
 	     * syntax names elsewhere.)  Skip over it here, but keep
 	     * track of its color regexes. */
 	    if (strcmp(tmpsyntax->desc, "default") == 0) {
-		defcolor = syntaxes->color;
+		defcolor = tmpsyntax->color;
 		continue;
 	    }
 
