@@ -1,4 +1,4 @@
-/* $Id: files.c,v 1.276 2006/02/24 19:38:20 dolorous Exp $ */
+/* $Id: files.c,v 1.294 2006/06/14 13:19:43 dolorous Exp $ */
 /**************************************************************************
  *   files.c                                                              *
  *                                                                        *
@@ -599,10 +599,11 @@ int open_file(const char *filename, bool newfie, FILE **f)
 	return -1;
     } else if (S_ISDIR(fileinfo.st_mode) || S_ISCHR(fileinfo.st_mode) ||
 	S_ISBLK(fileinfo.st_mode)) {
-	/* Don't open character or block files.  Sorry, /dev/sndstat! */
+	/* Don't open directories, character files, or block files.
+	 * Sorry, /dev/sndstat! */
 	statusbar(S_ISDIR(fileinfo.st_mode) ?
 		_("\"%s\" is a directory") :
-		_("File \"%s\" is a device file"), filename);
+		_("\"%s\" is a device file"), filename);
 	beep();
 	return -1;
     } else if ((fd = open(filename, O_RDONLY)) == -1) {
@@ -610,8 +611,7 @@ int open_file(const char *filename, bool newfie, FILE **f)
 	beep();
  	return -1;
      } else {
-	/* File is A-OK.  Open it in binary mode for our own end-of-line
-	 * character munging. */
+	/* The file is A-OK.  Open it. */
 	*f = fdopen(fd, "rb");
 
 	if (*f == NULL) {
@@ -687,27 +687,23 @@ void do_insertfile(
     bool at_edittop = FALSE;
 	/* Whether we're at the top of the edit window. */
 
-#ifndef DISABLE_WRAPPING
-    wrap_reset();
-#endif
-
     while (TRUE) {
 #ifndef NANO_TINY
 	if (execute) {
 	    msg = 
 #ifdef ENABLE_MULTIBUFFER
 		ISSET(MULTIBUFFER) ? 
-		N_("Command to execute in new buffer [from %s] ") :
+		_("Command to execute in new buffer [from %s] ") :
 #endif
-		N_("Command to execute [from %s] ");
+		_("Command to execute [from %s] ");
 	} else {
 #endif
 	    msg =
 #ifdef ENABLE_MULTIBUFFER
 		ISSET(MULTIBUFFER) ? 
-		N_("File to insert into new buffer [from %s] ") :
+		_("File to insert into new buffer [from %s] ") :
 #endif
-		N_("File to insert [from %s] ");
+		_("File to insert [from %s] ");
 #ifndef NANO_TINY
 	}
 #endif
@@ -723,10 +719,10 @@ void do_insertfile(
 #ifndef NANO_TINY
 		NULL,
 #endif
-		 edit_refresh, _(msg),
+		edit_refresh, msg,
 #ifndef DISABLE_OPERATINGDIR
-		operating_dir != NULL && strcmp(operating_dir, ".") != 0 ?
-		operating_dir :
+		operating_dir != NULL && strcmp(operating_dir,
+		".") != 0 ? operating_dir :
 #endif
 		"./");
 
@@ -1061,11 +1057,11 @@ char *safe_tempfile(FILE **f)
 
     assert(f != NULL);
 
-    /* If $TMPDIR is set and non-empty, set tempdir to it, run it
-     * through get_full_path(), and save the result in full_tempdir.
-     * Otherwise, leave full_tempdir set to NULL. */
+    /* If $TMPDIR is set, set tempdir to it, run it through
+     * get_full_path(), and save the result in full_tempdir.  Otherwise,
+     * leave full_tempdir set to NULL. */
     tmpdir_env = getenv("TMPDIR");
-    if (tmpdir_env != NULL && tmpdir_env[0] != '\0')
+    if (tmpdir_env != NULL)
 	full_tempdir = check_writable_directory(tmpdir_env);
 
     /* If $TMPDIR is unset, empty, or not a writable directory, and
@@ -1306,7 +1302,7 @@ int write_file(const char *name, FILE *f_open, bool tmp, append_type
 	goto cleanup_and_exit;
     }
 
-    /* Save the state of file at the end of the symlink (if there is
+    /* Save the state of the file at the end of the symlink (if there is
      * one). */
     realexists = (stat(realname, &st) != -1);
 
@@ -1336,7 +1332,10 @@ int write_file(const char *name, FILE *f_open, bool tmp, append_type
 		statusbar(_("Error reading %s: %s"), realname,
 			strerror(errno));
 		beep();
-		goto cleanup_and_exit;
+		/* If we can't read from the original file, go on, since
+		 * only saving the original file is better than saving
+		 * nothing. */
+		goto skip_backup;
 	    }
 	}
 
@@ -1375,8 +1374,10 @@ int write_file(const char *name, FILE *f_open, bool tmp, append_type
 		    _("Too many backup files?"));
 		free(backuptemp);
 		free(backupname);
-		fclose(f);
-		goto cleanup_and_exit;
+		/* If we can't write to the backup, go on, since only
+		 * saving the original file is better than saving
+		 * nothing. */
+		goto skip_backup;
 	    } else {
 		free(backupname);
 		backupname = backuptemp;
@@ -1398,8 +1399,9 @@ int write_file(const char *name, FILE *f_open, bool tmp, append_type
 	    free(backupname);
 	    if (backup_file != NULL)
 		fclose(backup_file);
-	    fclose(f);
-	    goto cleanup_and_exit;
+	    /* If we can't write to the backup, go on, since only saving
+	     * the original file is better than saving nothing. */
+	    goto skip_backup;
 	}
 
 #ifdef DEBUG
@@ -1409,12 +1411,11 @@ int write_file(const char *name, FILE *f_open, bool tmp, append_type
 	/* Copy the file. */
 	copy_status = copy_file(f, backup_file);
 
-	/* And set metadata. */
+	/* And set its metadata. */
 	if (copy_status != 0 || chown(backupname,
 		openfile->current_stat->st_uid,
 		openfile->current_stat->st_gid) == -1 ||
 		utime(backupname, &filetime) == -1) {
-	    free(backupname);
 	    if (copy_status == -1) {
 		statusbar(_("Error reading %s: %s"), realname,
 			strerror(errno));
@@ -1422,11 +1423,15 @@ int write_file(const char *name, FILE *f_open, bool tmp, append_type
 	    } else
 		statusbar(_("Error writing %s: %s"), backupname,
 			strerror(errno));
-	    goto cleanup_and_exit;
+	    /* If we can't read from or write to the backup, go on,
+	     * since only saving the original file is better than saving
+	     * nothing. */
 	}
 
 	free(backupname);
     }
+
+  skip_backup:
 #endif /* !NANO_TINY */
 
     /* If NOFOLLOW_SYMLINKS is set and the file is a link, we aren't
@@ -1454,10 +1459,21 @@ int write_file(const char *name, FILE *f_open, bool tmp, append_type
 	int fd_source;
 	FILE *f_source = NULL;
 
+	if (f == NULL) {
+	    f = fopen(realname, "rb");
+
+	    if (f == NULL) {
+		statusbar(_("Error reading %s: %s"), realname,
+			strerror(errno));
+		beep();
+		goto cleanup_and_exit;
+	    }
+	}
+
 	tempname = safe_tempfile(&f);
 
 	if (tempname == NULL) {
-	    statusbar(_("Prepending to %s failed: %s"), realname,
+	    statusbar(_("Error writing temp file: %s"),
 		strerror(errno));
 	    goto cleanup_and_exit;
 	}
@@ -1602,11 +1618,8 @@ int write_file(const char *name, FILE *f_open, bool tmp, append_type
 		strerror(errno));
 	    goto cleanup_and_exit;
 	}
-    } else if (fclose(f) == EOF) {
-	statusbar(_("Error writing %s: %s"), realname, strerror(errno));
-	unlink(tempname);
-	goto cleanup_and_exit;
-    }
+    } else
+	fclose(f);
 
     if (!tmp && append == OVERWRITE) {
 	if (!nonamechange) {
@@ -1735,21 +1748,21 @@ int do_writeout(bool exiting)
 	const char *formatstr, *backupstr;
 
 	formatstr = (openfile->fmt == DOS_FILE) ?
-		N_(" [DOS Format]") : (openfile->fmt == MAC_FILE) ?
-		N_(" [Mac Format]") : "";
+		_(" [DOS Format]") : (openfile->fmt == MAC_FILE) ?
+		_(" [Mac Format]") : "";
 
-	backupstr = ISSET(BACKUP_FILE) ? N_(" [Backup]") : "";
+	backupstr = ISSET(BACKUP_FILE) ? _(" [Backup]") : "";
 
 	if (openfile->mark_set && !exiting)
 	    msg = (append == PREPEND) ?
-		N_("Prepend Selection to File") : (append == APPEND) ?
-		N_("Append Selection to File") :
-		N_("Write Selection to File");
+		_("Prepend Selection to File") : (append == APPEND) ?
+		_("Append Selection to File") :
+		_("Write Selection to File");
 	else
 #endif /* !NANO_TINY */
-	    msg = (append == PREPEND) ? N_("File Name to Prepend to") :
-		(append == APPEND) ? N_("File Name to Append to") :
-		N_("File Name to Write");
+	    msg = (append == PREPEND) ? _("File Name to Prepend to") :
+		(append == APPEND) ? _("File Name to Append to") :
+		_("File Name to Write");
 
 	/* If we're using restricted mode, the filename isn't blank,
 	 * and we're at the "Write File" prompt, disable tab
@@ -1763,7 +1776,7 @@ int do_writeout(bool exiting)
 #ifndef NANO_TINY
 		NULL,
 #endif
-		 edit_refresh, "%s%s%s", _(msg),
+		edit_refresh, "%s%s%s", msg,
 #ifndef NANO_TINY
 		formatstr, backupstr
 #else
@@ -1820,8 +1833,14 @@ int do_writeout(bool exiting)
 #endif
 
 #ifdef NANO_EXTRA
-	    if (exiting && !ISSET(TEMP_FILE) &&
-		strcasecmp(answer, "zzy") == 0 && !did_credits) {
+	    /* If the current file has been modified, we've pressed
+	     * Ctrl-X at the edit window to exit, we've pressed "y" at
+	     * the "Save modified buffer" prompt to save, we've entered
+	     * "zzy" as the filename to save under (hence "xyzzy"), and
+	     * this is the first time we've done this, show an Easter
+	     * egg.  Display the credits. */
+	    if (!did_credits && exiting && !ISSET(TEMP_FILE) &&
+		strcasecmp(answer, "zzy") == 0) {
 		do_credits();
 		did_credits = TRUE;
 		retval = -1;
@@ -1868,7 +1887,7 @@ int do_writeout(bool exiting)
 
 	    break;
 	}
-    } /* while (TRUE) */
+    }
 
     free(ans);
 
@@ -1944,6 +1963,10 @@ int diralphasort(const void *va, const void *vb)
     if (!aisdir && bisdir)
 	return 1;
 
+    /* Standard function brain damage: We should be sorting
+     * alphabetically and case-insensitively according to the current
+     * locale, but there's no standard strcasecoll() function, so we
+     * have to use multibyte strcasecmp() instead, */
     return mbstrcasecmp(a, b);
 }
 
@@ -1951,6 +1974,8 @@ int diralphasort(const void *va, const void *vb)
  * elements. */
 void free_chararray(char **array, size_t len)
 {
+    assert(array != NULL);
+
     for (; len > 0; len--)
 	free(array[len - 1]);
     free(array);
@@ -2342,7 +2367,7 @@ void load_history(void)
 
     /* Assume do_rcfile() has reported a missing home directory. */
     if (nanohist != NULL) {
-	FILE *hist = fopen(nanohist, "r");
+	FILE *hist = fopen(nanohist, "rb");
 
 	if (hist == NULL) {
 	    if (errno != ENOENT) {
@@ -2351,7 +2376,7 @@ void load_history(void)
 		rcfile_error(N_("Error reading %s: %s"), nanohist,
 			strerror(errno));
 		fprintf(stderr,
-			_("\nPress Enter to continue starting nano\n"));
+			_("\nPress Enter to continue starting nano.\n"));
 		while (getchar() != '\n')
 		    ;
 	    }
