@@ -1,4 +1,4 @@
-/* $Id: help.c,v 1.9 2006/03/25 15:23:55 dolorous Exp $ */
+/* $Id: help.c,v 1.33 2006/06/19 16:04:42 dolorous Exp $ */
 /**************************************************************************
  *   help.c                                                               *
  *                                                                        *
@@ -32,28 +32,29 @@
 static char *help_text = NULL;
 	/* The text displayed in the help window. */
 
-/* Our dynamic, shortcut-list-compliant help function. */
-void do_help(void)
+/* Our dynamic, shortcut list-compliant help function.  refresh_func is
+ * the function we will call to refresh the edit window.*/
+void do_help(void (*refresh_func)(void))
 {
-    int line = 0;
+    size_t line = 0;
 	/* The line number in help_text of the first displayed help
 	 * line.  This variable is zero-based. */
-    bool no_more = FALSE;
-	/* no_more means the end of the help text is shown, so don't go
-	 * down any more. */
+    size_t last_line = 0;
+	/* The line number in help_text of the last help line.  This
+	 * variable is zero-based. */
     int kbinput = ERR;
     bool meta_key, func_key;
-
     bool old_no_help = ISSET(NO_HELP);
 #ifndef DISABLE_MOUSE
     const shortcut *oldshortcut = currshortcut;
 	/* We will set currshortcut to allow clicking on the help
 	 * screen's shortcut list. */
 #endif
+    const char *ptr;
 
     curs_set(0);
     blank_edit();
-    wattroff(bottomwin, A_REVERSE);
+    wattroff(bottomwin, reverse_attr);
     blank_statusbar();
 
     /* Set help_text as the string to display. */
@@ -76,11 +77,24 @@ void do_help(void)
 
     bottombars(help_list);
 
+    /* Get the last line of the help text. */
+    ptr = help_text;
+
+    for (; *ptr != '\0'; last_line++) {
+	ptr += help_line_len(ptr);
+	if (*ptr == '\n')
+	    ptr++;
+    }
+    if (last_line > 0)
+	last_line--;
+
     do {
-	int i;
-	int old_line = line;
+	size_t i;
+	    /* Generic loop variable. */
+	size_t old_line = line;
 	    /* We redisplay the help only if it moved. */
-	const char *ptr = help_text;
+
+	ptr = help_text;
 
 	switch (kbinput) {
 #ifndef DISABLE_MOUSE
@@ -91,17 +105,17 @@ void do_help(void)
 		}
 		break;
 #endif
+	    case NANO_REFRESH_KEY:
+		total_redraw();
+		break;
 	    case NANO_PREVPAGE_KEY:
-	    case NANO_PREVPAGE_FKEY:
-		if (line > 0) {
+		if (line > editwinrows - 2)
 		    line -= editwinrows - 2;
-		    if (line < 0)
-			line = 0;
-		}
+		else
+		    line = 0;
 		break;
 	    case NANO_NEXTPAGE_KEY:
-	    case NANO_NEXTPAGE_FKEY:
-		if (!no_more)
+		if (line + (editwinrows - 2) <= last_line)
 		    line += editwinrows - 2;
 		break;
 	    case NANO_PREVLINE_KEY:
@@ -109,20 +123,26 @@ void do_help(void)
 		    line--;
 		break;
 	    case NANO_NEXTLINE_KEY:
-		if (!no_more)
+		if (line + editwinrows <= last_line)
 		    line++;
 		break;
+	    case NANO_FIRSTLINE_ALTKEY:
+		if (meta_key)
+		    line = 0;
+		break;
+	    case NANO_LASTLINE_ALTKEY:
+		if (meta_key) {
+		    if (line + (editwinrows - 1) < last_line)
+			line = last_line - (editwinrows - 1);
+		}
+		break;
 	}
 
-	if (kbinput == NANO_REFRESH_KEY)
-	    /* Redraw the screen. */
-	    total_redraw();
-	else {
-	    if (kbinput != ERR && line == old_line)
-		goto skip_redisplay;
+	if ((kbinput != ERR && line == old_line) || kbinput ==
+		NANO_REFRESH_KEY)
+	    goto skip_redisplay;
 
-	    blank_edit();
-	}
+	blank_edit();
 
 	/* Calculate where in the text we should be, based on the
 	 * page. */
@@ -140,11 +160,11 @@ void do_help(void)
 	    if (*ptr == '\n')
 		ptr++;
 	}
-	no_more = (*ptr == '\0');
 
   skip_redisplay:
 	kbinput = get_kbinput(edit, &meta_key, &func_key);
-    } while (kbinput != NANO_EXIT_KEY && kbinput != NANO_EXIT_FKEY);
+	parse_help_input(&kbinput, &meta_key, &func_key);
+    } while (kbinput != NANO_EXIT_KEY);
 
 #ifndef DISABLE_MOUSE
     currshortcut = oldshortcut;
@@ -159,7 +179,7 @@ void do_help(void)
 	bottombars(currshortcut);
 
     curs_set(1);
-    edit_refresh();
+    refresh_func();
 
     /* The help_init() at the beginning allocated help_text.  Since 
      * help_text has now been written to the screen, we don't need it
@@ -168,6 +188,20 @@ void do_help(void)
     help_text = NULL;
 }
 
+/* Start the help browser for the edit window. */
+void do_help_void(void)
+{
+    do_help(&edit_refresh);
+}
+
+#ifndef DISABLE_BROWSER
+/* Start the help browser for the file browser. */
+void do_browser_help(void)
+{
+    do_help(&browser_refresh);
+}
+#endif
+
 /* This function allocates help_text, and stores the help string in it. 
  * help_text should be NULL initially. */
 void help_init(void)
@@ -256,6 +290,19 @@ void help_init(void)
 		"in the file browser:\n\n");
 	htx[1] = NULL;
 	htx[2] = NULL;
+    } else if (currshortcut == whereis_file_list) {
+	htx[0] = N_("Browser Search Command Help Text\n\n "
+		"Enter the words or characters you would like to "
+		"search for, and then press Enter.  If there is a "
+		"match for the text you entered, the screen will be "
+		"updated to the location of the nearest match for the "
+		"search string.\n\n The previous search string will be "
+		"shown in brackets after the search prompt.  Hitting "
+		"Enter without entering any text will perform the "
+		"previous search.\n\n");
+	htx[1] = N_(" The following function keys are available in "
+		"Browser Search mode:\n\n");
+	htx[2] = NULL;
     } else if (currshortcut == gotodir_list) {
 	htx[0] = N_("Browser Go To Directory Help Text\n\n "
 		"Enter the name of the directory you would like to "
@@ -277,7 +324,7 @@ void help_init(void)
 		"be edited.  It will then prompt to replace every "
 		"instance of the given misspelled word in the current "
 		"file, or, if you have selected text with the mark, in "
-		"the selected text.\n\n The following other functions "
+		"the selected text.\n\n The following function keys "
 		"are available in Spell Check mode:\n\n");
 	htx[1] = NULL;
 	htx[2] = NULL;
@@ -286,12 +333,12 @@ void help_init(void)
 #ifndef NANO_TINY
     else if (currshortcut == extcmd_list) {
 	htx[0] = N_("Execute Command Help Text\n\n "
-		"This menu allows you to insert the output of a "
+		"This mode allows you to insert the output of a "
 		"command run by the shell into the current buffer (or "
 		"a new buffer in multiple file buffer mode). If you "
 		"need another blank buffer, do not enter any "
-		"command.\n\n The following keys are available in "
-		"Execute Command mode:\n\n");
+		"command.\n\n The following function keys are "
+		"available in Execute Command mode:\n\n");
 	htx[1] = NULL;
 	htx[2] = NULL;
     }
@@ -308,8 +355,8 @@ void help_init(void)
 		"showing the file being edited.  The status line is "
 		"the third line from the bottom and shows important "
 		"messages.  The bottom two lines show the most "
-		"commonly used shortcuts in the editor.\n\n ");
-	htx[1] = N_("The notation for shortcuts is as follows: "
+		"commonly used shortcuts in the editor.\n\n");
+	htx[1] = N_(" The notation for shortcuts is as follows: "
 		"Control-key sequences are notated with a caret (^) "
 		"symbol and can be entered either by using the Control "
 		"(Ctrl) key or pressing the Escape (Esc) key twice.  "
@@ -336,20 +383,21 @@ void help_init(void)
     if (htx[2] != NULL)
 	allocsize += strlen(htx[2]);
 
-    /* The space needed for the shortcut lists, at most COLS characters,
-     * plus '\n'. */
-    allocsize += (COLS < 24 ? (24 * mb_cur_max()) :
-	((COLS + 1) * mb_cur_max())) * length_of_list(currshortcut);
+    /* Count the shortcut help text.  Each entry has up to three keys,
+     * which fill 24 columns, plus translated text, plus one or two
+     * \n's. */
+	for (s = currshortcut; s != NULL; s = s->next)
+	    allocsize += (24 * mb_cur_max()) + strlen(s->help) + 2;
 
 #ifndef NANO_TINY
     /* If we're on the main list, we also count the toggle help text.
-     * Each line has "M-%c\t\t\t", which fills 24 columns, plus a space,
-     * plus translated text, plus '\n'. */
+     * Each entry has "M-%c\t\t\t", which fills 24 columns, plus a
+     * space, plus translated text, plus one or two '\n's. */
     if (currshortcut == main_list) {
 	size_t endis_len = strlen(_("enable/disable"));
 
 	for (t = toggles; t != NULL; t = t->next)
-	    allocsize += 8 + strlen(t->desc) + endis_len;
+	    allocsize += strlen(t->desc) + endis_len + 9;
     }
 #endif
 
@@ -373,8 +421,8 @@ void help_init(void)
     /* Now add our shortcut info.  Assume that each shortcut has, at the
      * very least, an equivalent control key, an equivalent primary meta
      * key sequence, or both.  Also assume that the meta key values are
-     * not control characters.  We can display a maximum of 3 shortcut
-     * entries. */
+     * not control characters.  We can display a maximum of three
+     * shortcut entries. */
     for (s = currshortcut; s != NULL; s = s->next) {
 	int entries = 0;
 
@@ -384,9 +432,22 @@ void help_init(void)
 	    /* Yucky sentinel values that we can't handle a better
 	     * way. */
 	    if (s->ctrlval == NANO_CONTROL_SPACE) {
-		char *space_ptr = display_string(_("Space"), 0, 6,
+		char *space_ptr = display_string(_("Space"), 0, 14,
 			FALSE);
 
+		if (s->funcval == NANO_NO_KEY && (s->metaval ==
+			NANO_NO_KEY || s->miscval == NANO_NO_KEY)) {
+		    /* If we're here, we have at least two entries worth
+		     * of blank space.  If this entry takes up more than
+		     * one entry's worth of space, use two to display
+		     * it. */
+		    if (mbstrlen(space_ptr) > 6)
+			entries++;
+		} else
+		    /* Otherwise, truncate it so that it takes up only
+		     * one entry's worth of space. */
+		    space_ptr[6] = '\0';
+
 		ptr += sprintf(ptr, "^%s", space_ptr);
 
 		free(space_ptr);
@@ -422,9 +483,15 @@ void help_init(void)
 	    /* Yucky sentinel values that we can't handle a better
 	     * way. */
 	    if (s->metaval == NANO_ALT_SPACE && entries == 1) {
-		char *space_ptr = display_string(_("Space"), 0, 5,
+		char *space_ptr = display_string(_("Space"), 0, 13,
 			FALSE);
 
+		/* If we're here, we have at least two entries worth of
+		 * blank space.  If this entry takes up more than one
+		 * entry's worth of space, use two to display it. */
+		if (mbstrlen(space_ptr) > 5)
+		    entries++;
+
 		ptr += sprintf(ptr, "M-%s", space_ptr);
 
 		free(space_ptr);
@@ -447,34 +514,33 @@ void help_init(void)
 	    *(ptr++) = '\t';
 	}
 
-	/* Make sure all the help text starts at the same place. */
-	while (entries < 3) {
-	    entries++;
-	    *(ptr++) = '\t';
+	/* If this entry isn't blank, make sure all the help text starts
+	 * at the same place. */
+	if (s->ctrlval != NANO_NO_KEY || s->funcval != NANO_NO_KEY ||
+		s->metaval != NANO_NO_KEY || s->miscval !=
+		NANO_NO_KEY) {
+	    while (entries < 3) {
+		entries++;
+		*(ptr++) = '\t';
+	    }
 	}
 
-	assert(s->help != NULL);
+	/* The shortcut's help text. */
+	ptr += sprintf(ptr, "%s\n", s->help);
 
-	if (COLS > 24) {
-	    char *help_ptr = display_string(s->help, 0, COLS - 24,
-		FALSE);
-
-	    ptr += sprintf(ptr, help_ptr);
-
-	    free(help_ptr);
-	}
-
-	ptr += sprintf(ptr, "\n");
+	if (s->blank_after)
+	    ptr += sprintf(ptr, "\n");
     }
 
 #ifndef NANO_TINY
     /* And the toggles... */
     if (currshortcut == main_list) {
 	for (t = toggles; t != NULL; t = t->next) {
-	    assert(t->desc != NULL);
+	    ptr += sprintf(ptr, "M-%c\t\t\t%s %s\n",
+		toupper(t->val), t->desc, _("enable/disable"));
 
-	    ptr += sprintf(ptr, "M-%c\t\t\t%s %s\n", toupper(t->val),
-		t->desc, _("enable/disable"));
+	    if (t->blank_after)
+		ptr += sprintf(ptr, "\n");
 	}
     }
 
@@ -489,24 +555,51 @@ void help_init(void)
     assert(strlen(help_text) <= allocsize + 1);
 }
 
+/* Determine the shortcut key corresponding to the values of kbinput
+ * (the key itself), meta_key (whether the key is a meta sequence), and
+ * func_key (whether the key is a function key), if any.  In the
+ * process, convert certain non-shortcut keys into their corresponding
+ * shortcut keys. */
+void parse_help_input(int *kbinput, bool *meta_key, bool *func_key)
+{
+    get_shortcut(help_list, kbinput, meta_key, func_key);
+
+    if (*meta_key == FALSE) {
+	switch (*kbinput) {
+	    /* For consistency with the file browser. */
+	    case ' ':
+		*kbinput = NANO_NEXTPAGE_KEY;
+		break;
+	    case '-':
+		*kbinput = NANO_PREVPAGE_KEY;
+		break;
+	    /* Cancel is equivalent to Exit here. */
+	    case NANO_CANCEL_KEY:
+		*kbinput = NANO_EXIT_KEY;
+		break;
+	}
+    }
+}
+
 /* Calculate the next line of help_text, starting at ptr. */
 size_t help_line_len(const char *ptr)
 {
-    int help_cols = (COLS > 24) ? COLS - 8 : 24;
+    int help_cols = (COLS > 24) ? COLS - 1 : 24;
 
-    /* Try to break the line at (COLS - 8) columns if we have more than
+    /* Try to break the line at (COLS - 1) columns if we have more than
      * 24 columns, and at 24 columns otherwise. */
-    size_t retval = break_line(ptr, help_cols, TRUE);
+    ssize_t wrap_loc = break_line(ptr, help_cols, TRUE);
+    size_t retval = (wrap_loc < 0) ? 0 : wrap_loc;
     size_t retval_save = retval;
 
     /* Get the length of the entire line up to a null or a newline. */
     while (*(ptr + retval) != '\0' && *(ptr + retval) != '\n')
 	retval += move_mbright(ptr + retval, 0);
 
-    /* If the entire line doesn't go more than 8 columns beyond where we
+    /* If the entire line doesn't go more than 1 column beyond where we
      * tried to break it, we should display it as-is.  Otherwise, we
      * should display it only up to the break. */
-    if (strnlenpt(ptr, retval) > help_cols + 8)
+    if (strnlenpt(ptr, retval) > help_cols + 1)
 	retval = retval_save;
 
     return retval;
