@@ -1,9 +1,9 @@
-/* $Id: move.c,v 1.58 2005/12/08 07:09:08 dolorous Exp $ */
+/* $Id: move.c,v 1.61 2006/05/22 02:08:49 dolorous Exp $ */
 /**************************************************************************
  *   move.c                                                               *
  *                                                                        *
  *   Copyright (C) 1999-2004 Chris Allegretta                             *
- *   Copyright (C) 2005 David Lawrence Ramsey                             *
+ *   Copyright (C) 2005-2006 David Lawrence Ramsey                        *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
  *   the Free Software Foundation; either version 2, or (at your option)  *
@@ -32,8 +32,6 @@ void do_first_line(void)
     const filestruct *current_save = openfile->current;
     size_t pww_save = openfile->placewewant;
 
-    check_statusblank();
-
     openfile->current = openfile->fileage;
     openfile->current_x = 0;
     openfile->placewewant = 0;
@@ -47,8 +45,6 @@ void do_last_line(void)
     const filestruct *current_save = openfile->current;
     size_t pww_save = openfile->placewewant;
 
-    check_statusblank();
-
     openfile->current = openfile->filebot;
     openfile->current_x = strlen(openfile->filebot->data);
     openfile->placewewant = xplustabs();
@@ -62,12 +58,6 @@ void do_page_up(void)
 {
     int i;
 
-    check_statusblank();
-
-#ifndef DISABLE_WRAPPING
-    wrap_reset();
-#endif
-
     /* If there's less than a page of text left on the screen, put the
      * cursor at the beginning of the first line of the file, and then
      * update the edit window. */
@@ -103,12 +93,6 @@ void do_page_down(void)
 {
     int i;
 
-    check_statusblank();
-
-#ifndef DISABLE_WRAPPING
-    wrap_reset();
-#endif
-
     /* If there's less than a page of text left on the screen, put the
      * cursor at the beginning of the last line of the file, and then
      * update the edit window. */
@@ -149,8 +133,6 @@ void do_para_begin(bool allow_update)
     const filestruct *current_save = openfile->current;
     const size_t pww_save = openfile->placewewant;
 
-    check_statusblank();
-
     if (openfile->current != openfile->fileage) {
 	do {
 	    openfile->current = openfile->current->prev;
@@ -183,8 +165,6 @@ void do_para_end(bool allow_update)
     const filestruct *const current_save = openfile->current;
     const size_t pww_save = openfile->placewewant;
 
-    check_statusblank();
-
     while (openfile->current != openfile->filebot &&
 	!inpar(openfile->current))
 	openfile->current = openfile->current->next;
@@ -233,8 +213,6 @@ bool do_next_word(bool allow_punct, bool allow_update)
 
     assert(openfile->current != NULL && openfile->current->data != NULL);
 
-    check_statusblank();
-
     char_mb = charalloc(mb_cur_max());
 
     /* Move forward until we find the character after the last letter of
@@ -331,8 +309,6 @@ bool do_prev_word(bool allow_punct, bool allow_update)
 
     assert(openfile->current != NULL && openfile->current->data != NULL);
 
-    check_statusblank();
-
     char_mb = charalloc(mb_cur_max());
 
     /* Move backward until we find the character before the first letter
@@ -454,14 +430,12 @@ void do_prev_word_void(void)
 
 /* Move to the beginning of the current line.  If the SMART_HOME flag is
  * set, move to the first non-whitespace character of the current line
- * if we're not already there, or to the beginning of the current line
+ * if we aren't already there, or to the beginning of the current line
  * if we are. */
 void do_home(void)
 {
     size_t pww_save = openfile->placewewant;
 
-    check_statusblank();
-
 #ifndef NANO_TINY
     if (ISSET(SMART_HOME)) {
 	size_t current_x_save = openfile->current_x;
@@ -490,8 +464,6 @@ void do_end(void)
 {
     size_t pww_save = openfile->placewewant;
 
-    check_statusblank();
-
     openfile->current_x = strlen(openfile->current->data);
     openfile->placewewant = xplustabs();
 
@@ -502,12 +474,6 @@ void do_end(void)
 /* Move up one line. */
 void do_up(void)
 {
-    check_statusblank();
-
-#ifndef DISABLE_WRAPPING
-    wrap_reset();
-#endif
-
     /* If we're at the top of the file, get out. */
     if (openfile->current == openfile->fileage)
 	return;
@@ -542,12 +508,6 @@ void do_up(void)
 /* Scroll up one line without scrolling the cursor. */
 void do_scroll_up(void)
 {
-    check_statusblank();
-
-#ifndef DISABLE_WRAPPING
-    wrap_reset();
-#endif
-
     /* If the top of the file is onscreen, get out. */
     if (openfile->edittop == openfile->fileage)
 	return;
@@ -567,12 +527,6 @@ void do_scroll_up(void)
 /* Move down one line. */
 void do_down(void)
 {
-    check_statusblank();
-
-#ifndef DISABLE_WRAPPING
-    wrap_reset();
-#endif
-
     /* If we're at the bottom of the file, get out. */
     if (openfile->current == openfile->filebot)
 	return;
@@ -607,12 +561,6 @@ void do_down(void)
 /* Scroll down one line without scrolling the cursor. */
 void do_scroll_down(void)
 {
-    check_statusblank();
-
-#ifndef DISABLE_WRAPPING
-    wrap_reset();
-#endif
-
     /* If we're at the bottom of the file, get out. */
     if (openfile->current == openfile->filebot)
 	return;
@@ -634,8 +582,6 @@ void do_left(void)
 {
     size_t pww_save = openfile->placewewant;
 
-    check_statusblank();
-
     if (openfile->current_x > 0)
 	openfile->current_x = move_mbleft(openfile->current->data,
 		openfile->current_x);
@@ -655,8 +601,6 @@ void do_right(void)
 {
     size_t pww_save = openfile->placewewant;
 
-    check_statusblank();
-
     assert(openfile->current_x <= strlen(openfile->current->data));
 
     if (openfile->current->data[openfile->current_x] != '\0')
