@@ -1,4 +1,4 @@
-/* $Id: nano.c,v 1.510 2006/03/20 04:46:48 dolorous Exp $ */
+/* $Id: nano.c,v 1.555 2006/06/25 17:25:13 dolorous Exp $ */
 /**************************************************************************
  *   nano.c                                                               *
  *                                                                        *
@@ -30,7 +30,6 @@
 #include <string.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
-#include <sys/param.h>
 #include <errno.h>
 #include <ctype.h>
 #include <locale.h>
@@ -40,20 +39,15 @@
 #include <getopt.h>
 #endif
 
-#ifndef NANO_TINY
-#include <setjmp.h>
+#ifdef ENABLE_NANORC
+static bool no_rcfiles = FALSE;
+	/* Should we ignore all rcfiles? */
 #endif
-
 static struct termios oldterm;
 	/* The user's original terminal settings. */
 static struct sigaction act;
 	/* For all our fun signal handlers. */
 
-#ifndef NANO_TINY
-static sigjmp_buf jmpbuf;
-	/* Used to return to main() after a SIGWINCH. */
-#endif
-
 /* Create a new filestruct node.  Note that we specifically do not set
  * prevnode->next equal to the new line. */
 filestruct *make_new_node(filestruct *prevnode)
@@ -163,7 +157,7 @@ void renumber(filestruct *fileptr)
 {
     ssize_t line;
 
-    assert(fileptr != NULL && fileptr->prev != NULL);
+    assert(fileptr != NULL);
 
     line = (fileptr->prev == NULL) ? 0 : fileptr->prev->lineno;
 
@@ -537,13 +531,14 @@ void finish(void)
 	blank_statusbar();
 
     wrefresh(bottomwin);
+
     endwin();
 
     /* Restore the old terminal settings. */
     tcsetattr(0, TCSANOW, &oldterm);
 
 #if !defined(NANO_TINY) && defined(ENABLE_NANORC)
-    if (!ISSET(NO_RCFILE) && ISSET(HISTORYLOG))
+    if (!no_rcfiles && ISSET(HISTORYLOG))
 	save_history();
 #endif
 
@@ -560,7 +555,6 @@ void die(const char *msg, ...)
     va_list ap;
 
     endwin();
-    curses_ended = TRUE;
 
     /* Restore the old terminal settings. */
     tcsetattr(0, TCSANOW, &oldterm);
@@ -671,27 +665,40 @@ void window_init(void)
 }
 
 #ifndef DISABLE_MOUSE
-/* Initialize mouse support. */
+/* Disable mouse support. */
+void disable_mouse_support(void)
+{
+    mousemask(0, NULL);
+}
+
+/* Enable mouse support. */
+void enable_mouse_support(void)
+{
+    mousemask(BUTTON1_RELEASED, NULL);
+    mouseinterval(50);
+}
+
+/* Initialize mouse support.  Enable it if the USE_MOUSE flag is set,
+ * and disable it otherwise. */
 void mouse_init(void)
 {
-    if (ISSET(USE_MOUSE)) {
-	mousemask(BUTTON1_RELEASED, NULL);
-	mouseinterval(50);
-    } else
-	mousemask(0, NULL);
+    if (ISSET(USE_MOUSE))
+	enable_mouse_support();
+    else
+	disable_mouse_support();
 }
-#endif
+#endif /* !DISABLE_MOUSE */
 
 #ifdef HAVE_GETOPT_LONG
-#define print1opt(shortflag, longflag, desc) print1opt_full(shortflag, longflag, desc)
+#define print_opt(shortflag, longflag, desc) print_opt_full(shortflag, longflag, desc)
 #else
-#define print1opt(shortflag, longflag, desc) print1opt_full(shortflag, desc)
+#define print_opt(shortflag, longflag, desc) print_opt_full(shortflag, desc)
 #endif
 
 /* Print one usage string to the screen.  This cuts down on duplicate
  * strings to translate, and leaves out the parts that shouldn't be
  * translatable (the flag names). */
-void print1opt_full(const char *shortflag
+void print_opt_full(const char *shortflag
 #ifdef HAVE_GETOPT_LONG
 	, const char *longflag
 #endif
@@ -717,108 +724,111 @@ void print1opt_full(const char *shortflag
 /* Explain how to properly use nano and its command line options. */
 void usage(void)
 {
-#ifdef HAVE_GETOPT_LONG
+    printf(_("Usage: nano [OPTIONS] [[+LINE[,COLUMN]] FILE]...\n\n"));
     printf(
-	_("Usage: nano [+LINE,COLUMN] [GNU long option] [option] [file]\n\n"));
-    printf(_("Option\t\tLong option\t\tMeaning\n"));
+#ifdef HAVE_GETOPT_LONG
+	_("Option\t\tGNU long option\t\tMeaning\n")
 #else
-    printf(_("Usage: nano [+LINE,COLUMN] [option] [file]\n\n"));
-    printf(_("Option\t\tMeaning\n"));
+	_("Option\t\tMeaning\n")
 #endif
-
-    print1opt("-h, -?", "--help", N_("Show this message"));
-    print1opt(_("+LINE,COLUMN"), "",
+	);
+    print_opt("-h, -?", "--help", N_("Show this message"));
+    print_opt(_("+LINE[,COLUMN]"), "",
 	N_("Start at line LINE, column COLUMN"));
 #ifndef NANO_TINY
-    print1opt("-A", "--smarthome", N_("Enable smart home key"));
-    print1opt("-B", "--backup", N_("Save backups of existing files"));
-    print1opt(_("-C [dir]"), _("--backupdir=[dir]"),
+    print_opt("-A", "--smarthome", N_("Enable smart home key"));
+    print_opt("-B", "--backup", N_("Save backups of existing files"));
+    print_opt(_("-C <dir>"), _("--backupdir=<dir>"),
 	N_("Directory for saving unique backup files"));
-    print1opt("-E", "--tabstospaces",
+#endif
+    print_opt("-D", "--boldtext",
+	N_("Use bold instead of reverse video text"));
+#ifndef NANO_TINY
+    print_opt("-E", "--tabstospaces",
 	N_("Convert typed tabs to spaces"));
 #endif
 #ifdef ENABLE_MULTIBUFFER
-    print1opt("-F", "--multibuffer", N_("Enable multiple file buffers"));
+    print_opt("-F", "--multibuffer", N_("Enable multiple file buffers"));
 #endif
 #ifdef ENABLE_NANORC
 #ifndef NANO_TINY
-    print1opt("-H", "--historylog",
+    print_opt("-H", "--historylog",
 	N_("Log & read search/replace string history"));
 #endif
-    print1opt("-I", "--ignorercfiles",
+    print_opt("-I", "--ignorercfiles",
 	N_("Don't look at nanorc files"));
 #endif
-    print1opt("-K", "--rebindkeypad",
+    print_opt("-K", "--rebindkeypad",
 	N_("Fix numeric keypad key confusion problem"));
-    print1opt("-L", "--nonewlines",
+    print_opt("-L", "--nonewlines",
 	N_("Don't add newlines to the ends of files"));
 #ifndef NANO_TINY
-    print1opt("-N", "--noconvert",
+    print_opt("-N", "--noconvert",
 	N_("Don't convert files from DOS/Mac format"));
 #endif
-    print1opt("-O", "--morespace", N_("Use more space for editing"));
+    print_opt("-O", "--morespace", N_("Use more space for editing"));
 #ifndef DISABLE_JUSTIFY
-    print1opt(_("-Q [str]"), _("--quotestr=[str]"),
+    print_opt(_("-Q <str>"), _("--quotestr=<str>"),
 	N_("Quoting string"));
 #endif
-    print1opt("-R", "--restricted", N_("Restricted mode"));
+    print_opt("-R", "--restricted", N_("Restricted mode"));
 #ifndef NANO_TINY
-    print1opt("-S", "--smooth", N_("Smooth scrolling"));
+    print_opt("-S", "--smooth", N_("Smooth scrolling"));
 #endif
-    print1opt(_("-T [#cols]"), _("--tabsize=[#cols]"),
+    print_opt(_("-T <#cols>"), _("--tabsize=<#cols>"),
 	N_("Set width of a tab in cols to #cols"));
 #ifndef NANO_TINY
-    print1opt("-U", "--quickblank", N_("Do quick statusbar blanking"));
+    print_opt("-U", "--quickblank", N_("Do quick statusbar blanking"));
 #endif
-    print1opt("-V", "--version",
+    print_opt("-V", "--version",
 	N_("Print version information and exit"));
 #ifndef NANO_TINY
-    print1opt("-W", "--wordbounds",
+    print_opt("-W", "--wordbounds",
 	N_("Detect word boundaries more accurately"));
 #endif
 #ifdef ENABLE_COLOR
-    print1opt(_("-Y [str]"), _("--syntax=[str]"),
+    print_opt(_("-Y <str>"), _("--syntax=<str>"),
 	N_("Syntax definition to use"));
 #endif
-    print1opt("-c", "--const", N_("Constantly show cursor position"));
-    print1opt("-d", "--rebinddelete",
+    print_opt("-c", "--const", N_("Constantly show cursor position"));
+    print_opt("-d", "--rebinddelete",
 	N_("Fix Backspace/Delete confusion problem"));
 #ifndef NANO_TINY
-    print1opt("-i", "--autoindent",
+    print_opt("-i", "--autoindent",
 	N_("Automatically indent new lines"));
-    print1opt("-k", "--cut", N_("Cut from cursor to end of line"));
+    print_opt("-k", "--cut", N_("Cut from cursor to end of line"));
 #endif
-    print1opt("-l", "--nofollow",
+    print_opt("-l", "--nofollow",
 	N_("Don't follow symbolic links, overwrite"));
 #ifndef DISABLE_MOUSE
-    print1opt("-m", "--mouse", N_("Enable mouse"));
+    print_opt("-m", "--mouse", N_("Enable the use of the mouse"));
 #endif
 #ifndef DISABLE_OPERATINGDIR
-    print1opt(_("-o [dir]"), _("--operatingdir=[dir]"),
+    print_opt(_("-o <dir>"), _("--operatingdir=<dir>"),
 	N_("Set operating directory"));
 #endif
-    print1opt("-p", "--preserve",
+    print_opt("-p", "--preserve",
 	N_("Preserve XON (^Q) and XOFF (^S) keys"));
 #ifndef DISABLE_WRAPJUSTIFY
-    print1opt(_("-r [#cols]"), _("--fill=[#cols]"),
+    print_opt(_("-r <#cols>"), _("--fill=<#cols>"),
 	N_("Set fill cols to (wrap lines at) #cols"));
 #endif
 #ifndef DISABLE_SPELLER
-    print1opt(_("-s [prog]"), _("--speller=[prog]"),
+    print_opt(_("-s <prog>"), _("--speller=<prog>"),
 	N_("Enable alternate speller"));
 #endif
-    print1opt("-t", "--tempfile",
+    print_opt("-t", "--tempfile",
 	N_("Auto save on exit, don't prompt"));
-    print1opt("-v", "--view", N_("View (read only) mode"));
+    print_opt("-v", "--view", N_("View (read only) mode"));
 #ifndef DISABLE_WRAPPING
-    print1opt("-w", "--nowrap", N_("Don't wrap long lines"));
+    print_opt("-w", "--nowrap", N_("Don't wrap long lines"));
 #endif
-    print1opt("-x", "--nohelp", N_("Don't show help window"));
-    print1opt("-z", "--suspend", N_("Enable suspend"));
+    print_opt("-x", "--nohelp", N_("Don't show the two help lines"));
+    print_opt("-z", "--suspend", N_("Enable suspension"));
 
     /* This is a special case. */
-    print1opt("-a, -b, -e,", "", NULL);
-    print1opt("-f, -g, -j", "", N_("(ignored, for Pico compatibility)"));
+    print_opt("-a, -b, -e,", "", NULL);
+    print_opt("-f, -g, -j", "", N_("(ignored, for Pico compatibility)"));
 
     exit(0);
 }
@@ -996,8 +1006,18 @@ RETSIGTYPE handle_hupterm(int signal)
 /* Handler for SIGTSTP (suspend). */
 RETSIGTYPE do_suspend(int signal)
 {
-    endwin();
-    printf("\n\n\n\n\n%s\n", _("Use \"fg\" to return to nano"));
+#ifndef DISABLE_MOUSE
+    /* Turn mouse support off. */
+    disable_mouse_support();
+#endif
+
+    /* Blank the screen, and move the cursor to the last line of it. */
+    erase();
+    move(LINES - 1, 0);
+    refresh();
+
+    /* Display our helpful message. */
+    printf(_("Use \"fg\" to return to nano.\n"));
     fflush(stdout);
 
     /* Restore the old terminal settings. */
@@ -1016,13 +1036,27 @@ RETSIGTYPE do_suspend(int signal)
 /* Handler for SIGCONT (continue after suspend). */
 RETSIGTYPE do_continue(int signal)
 {
+#ifndef DISABLE_MOUSE
+    /* Turn mouse support back on if it was on before. */
+    if (ISSET(USE_MOUSE))
+	enable_mouse_support();
+#endif
+
 #ifndef NANO_TINY
     /* Perhaps the user resized the window while we slept.  Handle it,
-     * and update the screen in the process. */
+     * and restore the terminal to its previous state and update the
+     * screen in the process. */
     handle_sigwinch(0);
 #else
-    /* Reenter curses mode, and update the screen in the process. */
-    doupdate();
+    /* Restore the terminal to its previous state. */
+    terminal_init();
+
+    /* Turn the cursor back on for sure. */
+    curs_set(1);
+
+    /* Redraw the contents of the windows that need it. */
+    blank_statusbar();
+    total_refresh();
 #endif
 }
 
@@ -1055,16 +1089,6 @@ RETSIGTYPE handle_sigwinch(int signal)
     if (filepart != NULL)
 	unpartition_filestruct(&filepart);
 
-#ifndef DISABLE_JUSTIFY
-    /* If the justify buffer isn't empty, blow away the text in it and
-     * display the shortcut list with UnCut. */
-    if (jusbuffer != NULL) {
-	free_filestruct(jusbuffer);
-	jusbuffer = NULL;
-	shortcut_init(FALSE);
-    }
-#endif
-
 #ifdef USE_SLANG
     /* Slang curses emulation brain damage, part 1: If we just do what
      * curses does here, it'll only work properly if the resize made the
@@ -1095,11 +1119,9 @@ RETSIGTYPE handle_sigwinch(int signal)
     currshortcut = main_list;
     total_refresh();
 
-    /* Reset all the input routines that rely on character sequences. */
-    reset_kbinput();
-
-    /* Jump back to the main loop. */
-    siglongjmp(jmpbuf, 1);
+    /* Jump back to either main() or the unjustify routine in
+     * do_justify(). */
+    siglongjmp(jump_buf, 1);
 }
 
 /* If allow is TRUE, block any SIGWINCH signals that we get, so that we
@@ -1234,13 +1256,32 @@ void enable_flow_control(void)
  * interpretation of the flow control characters too. */
 void terminal_init(void)
 {
-    cbreak();
-    nonl();
-    noecho();
-    disable_extended_io();
-    disable_signals();
-    if (!ISSET(PRESERVE))
-	disable_flow_control();
+#ifdef USE_SLANG
+    /* Slang curses emulation brain damage, part 2: Slang doesn't
+     * implement nonl() or noecho() properly, so there's no way to
+     * properly reinitialize the terminal using them.  We have to save
+     * the terminal state after the first call and restore it on
+     * subsequent calls. */
+    static struct termios newterm;
+    static bool newterm_set = FALSE;
+
+    if (!newterm_set) {
+#endif
+
+	cbreak();
+	nonl();
+	noecho();
+	disable_extended_io();
+	disable_signals();
+	if (!ISSET(PRESERVE))
+	    disable_flow_control();
+
+#ifdef USE_SLANG
+	tcgetattr(0, &newterm);
+	newterm_set = TRUE;
+    } else
+	tcsetattr(0, TCSANOW, &newterm);
+#endif
 }
 
 /* Read in a character, interpret it as a shortcut or toggle if
@@ -1261,6 +1302,8 @@ int do_input(bool *meta_key, bool *func_key, bool *s_or_t, bool
 	/* The input buffer. */
     static size_t kbinput_len = 0;
 	/* The length of the input buffer. */
+    bool cut_copy = FALSE;
+	/* Are we cutting or copying text? */
     const shortcut *s;
     bool have_shortcut;
 #ifndef NANO_TINY
@@ -1276,13 +1319,18 @@ int do_input(bool *meta_key, bool *func_key, bool *s_or_t, bool
     input = get_kbinput(edit, meta_key, func_key);
 
 #ifndef DISABLE_MOUSE
-    /* If we got a mouse click and it was on a shortcut, read in the
-     * shortcut character. */
-    if (allow_funcs && *func_key == TRUE && input == KEY_MOUSE) {
-	if (do_mouse())
-	    input = get_kbinput(edit, meta_key, func_key);
-	else
-	    input = ERR;
+    if (allow_funcs) {
+	/* If we got a mouse click and it was on a shortcut, read in the
+	 * shortcut character. */
+	if (*func_key == TRUE && input == KEY_MOUSE) {
+	    if (do_mouse())
+		input = get_kbinput(edit, meta_key, func_key);
+	    else {
+		*meta_key = FALSE;
+		*func_key = FALSE;
+		input = ERR;
+	    }
+	}
     }
 #endif
 
@@ -1310,6 +1358,19 @@ int do_input(bool *meta_key, bool *func_key, bool *s_or_t, bool
 #endif
 	);
 
+    /* If we got a non-high-bit control key, a meta key sequence, or a
+     * function key, and it's not a shortcut or toggle, throw it out. */
+    if (*s_or_t == FALSE) {
+	if (is_ascii_cntrl_char(input) || *meta_key == TRUE ||
+		*func_key == TRUE) {
+	    statusbar(_("Unknown Command"));
+	    beep();
+	    *meta_key = FALSE;
+	    *func_key = FALSE;
+	    input = ERR;
+	}
+    }
+
     if (allow_funcs) {
 	/* If we got a character, and it isn't a shortcut or toggle,
 	 * it's a normal text character.  Display the warning if we're
@@ -1328,10 +1389,19 @@ int do_input(bool *meta_key, bool *func_key, bool *s_or_t, bool
 
 	/* If we got a shortcut or toggle, or if there aren't any other
 	 * characters waiting after the one we read in, we need to
-	 * display all the characters in the input buffer if it isn't
+	 * output all the characters in the input buffer if it isn't
 	 * empty.  Note that it should be empty if we're in view
 	 * mode. */
 	 if (*s_or_t == TRUE || get_key_buffer_len() == 0) {
+#ifndef DISABLE_WRAPPING
+	    /* If we got a shortcut or toggle, and it's not the shortcut
+	     * for verbatim input, turn off prepending of wrapped
+	     * text. */
+	    if (*s_or_t == TRUE && (!have_shortcut || s == NULL ||
+		s->func != do_verbatim_input))
+		wrap_reset();
+#endif
+
 	    if (kbinput != NULL) {
 		/* Display all the characters in the input buffer at
 		 * once, filtering out control characters. */
@@ -1357,27 +1427,30 @@ int do_input(bool *meta_key, bool *func_key, bool *s_or_t, bool
 	    switch (input) {
 		/* Handle the "universal" edit window shortcuts. */
 		case NANO_XON_KEY:
-		    statusbar(_("XON ignored, mumble mumble."));
+		    statusbar(_("XON ignored, mumble mumble"));
 		    break;
 		case NANO_XOFF_KEY:
-		    statusbar(_("XOFF ignored, mumble mumble."));
+		    statusbar(_("XOFF ignored, mumble mumble"));
 		    break;
-#ifndef NANO_TINY
 		case NANO_SUSPEND_KEY:
 		    if (ISSET(SUSPEND))
 			do_suspend(0);
 		    break;
-#endif
 		/* Handle the normal edit window shortcuts, setting
 		 * ran_func to TRUE if we try to run their associated
 		 * functions and setting finished to TRUE to indicate
 		 * that we're done after running or trying to run their
 		 * associated functions. */
 		default:
-		    /* Blow away the text in the cutbuffer if we aren't
-		     * cutting text. */
-		    if (s->func != do_cut_text)
-			cutbuffer_reset();
+		    /* If the function associated with this shortcut is
+		     * cutting or copying text, indicate this. */
+		    if (s->func == do_cut_text_void
+#ifndef NANO_TINY
+			|| s->func == do_copy_text || s->func ==
+			do_cut_till_end
+#endif
+			)
+			cut_copy = TRUE;
 
 		    if (s->func != NULL) {
 			*ran_func = TRUE;
@@ -1392,20 +1465,18 @@ int do_input(bool *meta_key, bool *func_key, bool *s_or_t, bool
 	}
 #ifndef NANO_TINY
 	else if (have_toggle) {
-	    /* Blow away the text in the cutbuffer, since we aren't
-	     * cutting text. */
-	    cutbuffer_reset();
 	    /* Toggle the flag associated with this shortcut. */
 	    if (allow_funcs)
 		do_toggle(t);
 	}
 #endif
-	else
-	    /* Blow away the text in the cutbuffer, since we aren't
-	     * cutting text. */
-	    cutbuffer_reset();
     }
 
+    /* If we aren't cutting or copying text, blow away the text in the
+     * cutbuffer. */
+    if (!cut_copy)
+	cutbuffer_reset();
+
     return input;
 }
 
@@ -1461,15 +1532,15 @@ bool do_mouse(void)
 }
 #endif /* !DISABLE_MOUSE */
 
-/* The user typed ouuput_len multibyte characters.  Add them to the edit
- * buffer, filtering out all control characters if allow_cntrls is
+/* The user typed output_len multibyte characters.  Add them to the edit
+ * buffer, filtering out all ASCII control characters if allow_cntrls is
  * TRUE. */
 void do_output(char *output, size_t output_len, bool allow_cntrls)
 {
     size_t current_len, i = 0;
     bool do_refresh = FALSE;
 	/* Do we have to call edit_refresh(), or can we get away with
-	 * update_line()? */
+	 * just update_line()? */
 
     char *char_buf = charalloc(mb_cur_max());
     int char_buf_len;
@@ -1480,7 +1551,7 @@ void do_output(char *output, size_t output_len, bool allow_cntrls)
 
     while (i < output_len) {
 	/* If allow_cntrls is FALSE, filter out nulls and newlines,
-	 * since they're control characters. */
+	 * since they're ASCII control characters. */
 	if (allow_cntrls) {
 	    /* Null to newline, if needed. */
 	    if (output[i] == '\0')
@@ -1498,8 +1569,10 @@ void do_output(char *output, size_t output_len, bool allow_cntrls)
 
 	i += char_buf_len;
 
-	/* If allow_cntrls is FALSE, filter out a control character. */
-	if (!allow_cntrls && is_cntrl_mbchar(output + i - char_buf_len))
+	/* If allow_cntrls is FALSE, filter out an ASCII control
+	 * character. */
+	if (!allow_cntrls && is_ascii_cntrl_char(*(output + i -
+		char_buf_len)))
 	    continue;
 
 	/* If the NO_NEWLINES flag isn't set, when a character is
@@ -1585,6 +1658,7 @@ int main(int argc, char **argv)
 #ifdef HAVE_GETOPT_LONG
     const struct option long_options[] = {
 	{"help", 0, NULL, 'h'},
+	{"boldtext", 0, NULL, 'D'},
 #ifdef ENABLE_MULTIBUFFER
 	{"multibuffer", 0, NULL, 'F'},
 #endif
@@ -1651,10 +1725,10 @@ int main(int argc, char **argv)
 
 	if (locale != NULL && (strcasestr(locale, "UTF8") != NULL ||
 		strcasestr(locale, "UTF-8") != NULL)) {
-	    SET(USE_UTF8);
 #ifdef USE_SLANG
 	    SLutf8_enable(TRUE);
 #endif
+	    utf8_init();
 	}
     }
 #else
@@ -1676,11 +1750,11 @@ int main(int argc, char **argv)
     while ((optchr =
 #ifdef HAVE_GETOPT_LONG
 	getopt_long(argc, argv,
-		"h?ABC:EFHIKLNOQ:RST:UVWY:abcdefgijklmo:pr:s:tvwxz",
+		"h?ABC:DEFHIKLNOQ:RST:UVWY:abcdefgijklmo:pr:s:tvwxz",
 		long_options, NULL)
 #else
 	getopt(argc, argv,
-		"h?ABC:EFHIKLNOQ:RST:UVWY:abcdefgijklmo:pr:s:tvwxz")
+		"h?ABC:DEFHIKLNOQ:RST:UVWY:abcdefgijklmo:pr:s:tvwxz")
 #endif
 		) != -1) {
 	switch (optchr) {
@@ -1702,6 +1776,11 @@ int main(int argc, char **argv)
 	    case 'C':
 		backup_dir = mallocstrcpy(backup_dir, optarg);
 		break;
+#endif
+	    case 'D':
+		SET(BOLD_TEXT);
+		break;
+#ifndef NANO_TINY
 	    case 'E':
 		SET(TABS_TO_SPACES);
 		break;
@@ -1718,7 +1797,7 @@ int main(int argc, char **argv)
 		break;
 #endif
 	    case 'I':
-		SET(NO_RCFILE);
+		no_rcfiles = TRUE;
 		break;
 #endif
 	    case 'K':
@@ -1750,7 +1829,7 @@ int main(int argc, char **argv)
 #endif
 	    case 'T':
 		if (!parse_num(optarg, &tabsize) || tabsize <= 0) {
-		    fprintf(stderr, _("Requested tab size %s invalid"), optarg);
+		    fprintf(stderr, _("Requested tab size \"%s\" is invalid"), optarg);
 		    fprintf(stderr, "\n");
 		    exit(1);
 		}
@@ -1806,7 +1885,7 @@ int main(int argc, char **argv)
 #ifndef DISABLE_WRAPJUSTIFY
 	    case 'r':
 		if (!parse_num(optarg, &wrap_at)) {
-		    fprintf(stderr, _("Requested fill size %s invalid"), optarg);
+		    fprintf(stderr, _("Requested fill size \"%s\" is invalid"), optarg);
 		    fprintf(stderr, "\n");
 		    exit(1);
 		}
@@ -1851,14 +1930,16 @@ int main(int argc, char **argv)
     if (ISSET(RESTRICTED)) {
 	UNSET(SUSPEND);
 	UNSET(BACKUP_FILE);
-	SET(NO_RCFILE);
+#ifdef ENABLE_NANORC
+	no_rcfiles = TRUE;
+#endif
     }
 
 /* We've read through the command line options.  Now back up the flags
  * and values that are set, and read the rcfile(s).  If the values
  * haven't changed afterward, restore the backed-up values. */
 #ifdef ENABLE_NANORC
-    if (!ISSET(NO_RCFILE)) {
+    if (!no_rcfiles) {
 #ifndef DISABLE_OPERATINGDIR
 	char *operating_dir_cpy = operating_dir;
 #endif
@@ -1930,10 +2011,16 @@ int main(int argc, char **argv)
 #endif
 #endif /* ENABLE_NANORC */
 
+    /* If we're using bold text instead of reverse video text, set it up
+     * now. */
+    if (ISSET(BOLD_TEXT))
+	reverse_attr = A_BOLD;
+
 #ifndef NANO_TINY
+    /* Set up the search/replace history. */
     history_init();
 #ifdef ENABLE_NANORC
-    if (!ISSET(NO_RCFILE) && ISSET(HISTORYLOG))
+    if (!no_rcfiles && ISSET(HISTORYLOG))
 	load_history();
 #endif
 #endif
@@ -2049,6 +2136,7 @@ int main(int argc, char **argv)
     shortcut_init(FALSE);
 
 #ifndef DISABLE_MOUSE
+    /* Initialize mouse support. */
     mouse_init();
 #endif
 
@@ -2125,11 +2213,6 @@ int main(int argc, char **argv)
 
     display_main_list();
 
-#ifndef NANO_TINY
-    /* Return here after a SIGWINCH. */
-    sigsetjmp(jmpbuf, 1);
-#endif
-
     display_buffer();    
 
     while (TRUE) {
@@ -2138,6 +2221,17 @@ int main(int argc, char **argv)
 	/* Make sure the cursor is in the edit window. */
 	reset_cursor();
 
+#ifndef NANO_TINY
+	if (!jump_buf_main) {
+	    /* If we haven't already, we're going to set jump_buf so
+	     * that we return here after a SIGWINCH.  Indicate this. */
+	    jump_buf_main = TRUE;
+
+	    /* Return here after a SIGWINCH. */
+	    sigsetjmp(jump_buf, 1);
+	}
+#endif
+
 	/* If constant cursor position display is on, and there are no
 	 * keys waiting in the input buffer, display the current cursor
 	 * position on the statusbar. */
