@@ -1,4 +1,4 @@
-/* $Id: prompt.c,v 1.34 2006/02/18 21:32:29 dolorous Exp $ */
+/* $Id: prompt.c,v 1.60 2006/06/03 19:36:02 dolorous Exp $ */
 /**************************************************************************
  *   prompt.c                                                             *
  *                                                                        *
@@ -28,15 +28,14 @@
 #include <string.h>
 
 static char *prompt = NULL;
-				/* The prompt string for statusbar
-				 * questions. */
+	/* The prompt string for statusbar questions. */
 static size_t statusbar_x = (size_t)-1;
-				/* The cursor position in answer. */
+	/* The cursor position in answer. */
 static size_t statusbar_pww = (size_t)-1;
-				/* The place we want in answer. */
+	/* The place we want in answer. */
 static bool reset_statusbar_x = FALSE;
-				/* Should we reset the cursor position
-				 * at the statusbar prompt? */
+	/* Should we reset the cursor position at the statusbar
+	 * prompt? */
 
 /* Read in a character, interpret it as a shortcut or toggle if
  * necessary, and return it.  Set meta_key to TRUE if the character is a
@@ -69,13 +68,18 @@ int do_statusbar_input(bool *meta_key, bool *func_key, bool *s_or_t,
     input = get_kbinput(bottomwin, meta_key, func_key);
 
 #ifndef DISABLE_MOUSE
-    /* If we got a mouse click and it was on a shortcut, read in the
-     * shortcut character. */
-    if (allow_funcs && *func_key == TRUE && input == KEY_MOUSE) {
-	if (do_statusbar_mouse())
-	    input = get_kbinput(bottomwin, meta_key, func_key);
-	else
-	    input = ERR;
+    if (allow_funcs) {
+	/* If we got a mouse click and it was on a shortcut, read in the
+	 * shortcut character. */
+	if (*func_key == TRUE && input == KEY_MOUSE) {
+	    if (do_statusbar_mouse())
+		input = get_kbinput(bottomwin, meta_key, func_key);
+	    else {
+		*meta_key = FALSE;
+		*func_key = FALSE;
+		input = ERR;
+	    }
+	}
     }
 #endif
 
@@ -84,27 +88,36 @@ int do_statusbar_input(bool *meta_key, bool *func_key, bool *s_or_t,
 
     /* If we got a shortcut from the current list, or a "universal"
      * statusbar prompt shortcut, set have_shortcut to TRUE. */
-    have_shortcut = (s != NULL || input == NANO_REFRESH_KEY ||
-	input == NANO_HOME_KEY || input == NANO_END_KEY ||
-	input == NANO_BACK_KEY || input == NANO_FORWARD_KEY ||
-	input == NANO_BACKSPACE_KEY || input == NANO_DELETE_KEY ||
-	input == NANO_CUT_KEY ||
+    have_shortcut = (s != NULL || input == NANO_TAB_KEY || input ==
+	NANO_ENTER_KEY || input == NANO_REFRESH_KEY || input ==
+	NANO_HOME_KEY || input == NANO_END_KEY || input ==
+	NANO_BACK_KEY || input == NANO_FORWARD_KEY || input ==
+	NANO_BACKSPACE_KEY || input == NANO_DELETE_KEY || input ==
+	NANO_CUT_KEY ||
 #ifndef NANO_TINY
-		input == NANO_NEXTWORD_KEY ||
+	input == NANO_NEXTWORD_KEY ||
 #endif
-		(*meta_key == TRUE && (
+	(*meta_key == TRUE && (
 #ifndef NANO_TINY
-		input == NANO_PREVWORD_KEY ||
+	input == NANO_PREVWORD_KEY || input == NANO_BRACKET_KEY ||
 #endif
-		input == NANO_VERBATIM_KEY
-#ifndef NANO_TINY
-		|| input == NANO_BRACKET_KEY
-#endif
-		)));
+	input == NANO_VERBATIM_KEY)));
 
     /* Set s_or_t to TRUE if we got a shortcut. */
     *s_or_t = have_shortcut;
 
+    /* If we got a non-high-bit control key, a meta key sequence, or a
+     * function key, and it's not a shortcut or toggle, throw it out. */
+    if (*s_or_t == FALSE) {
+	if (is_ascii_cntrl_char(input) || *meta_key == TRUE ||
+		*func_key == TRUE) {
+	    beep();
+	    *meta_key = FALSE;
+	    *func_key = FALSE;
+	    input = ERR;
+	}
+    }
+
     if (allow_funcs) {
 	/* If we got a character, and it isn't a shortcut or toggle,
 	 * it's a normal text character.  Display the warning if we're
@@ -154,36 +167,11 @@ int do_statusbar_input(bool *meta_key, bool *func_key, bool *s_or_t,
 	if (have_shortcut) {
 	    switch (input) {
 		/* Handle the "universal" statusbar prompt shortcuts. */
+		case NANO_TAB_KEY:
+		case NANO_ENTER_KEY:
+		    break;
 		case NANO_REFRESH_KEY:
-		    refresh_func();
-		    break;
-		case NANO_HOME_KEY:
-		    do_statusbar_home();
-		    break;
-		case NANO_END_KEY:
-		    do_statusbar_end();
-		    break;
-		case NANO_BACK_KEY:
-		    do_statusbar_left();
-		    break;
-		case NANO_FORWARD_KEY:
-		    do_statusbar_right();
-		    break;
-		case NANO_BACKSPACE_KEY:
-		    /* If we're using restricted mode, the filename
-		     * isn't blank, and we're at the "Write File"
-		     * prompt, disable Backspace. */
-		    if (!ISSET(RESTRICTED) || openfile->filename[0] ==
-			'\0' || currshortcut != writefile_list)
-			do_statusbar_backspace();
-		    break;
-		case NANO_DELETE_KEY:
-		    /* If we're using restricted mode, the filename
-		     * isn't blank, and we're at the "Write File"
-		     * prompt, disable Delete. */
-		    if (!ISSET(RESTRICTED) || openfile->filename[0] ==
-			'\0' || currshortcut != writefile_list)
-			do_statusbar_delete();
+		    total_statusbar_refresh(refresh_func);
 		    break;
 		case NANO_CUT_KEY:
 		    /* If we're using restricted mode, the filename
@@ -193,6 +181,12 @@ int do_statusbar_input(bool *meta_key, bool *func_key, bool *s_or_t,
 			'\0' || currshortcut != writefile_list)
 			do_statusbar_cut_text();
 		    break;
+		case NANO_FORWARD_KEY:
+		    do_statusbar_right();
+		    break;
+		case NANO_BACK_KEY:
+		    do_statusbar_left();
+		    break;
 #ifndef NANO_TINY
 		case NANO_NEXTWORD_KEY:
 		    do_statusbar_next_word(FALSE);
@@ -201,6 +195,18 @@ int do_statusbar_input(bool *meta_key, bool *func_key, bool *s_or_t,
 		    if (*meta_key == TRUE)
 			do_statusbar_prev_word(FALSE);
 		    break;
+#endif
+		case NANO_HOME_KEY:
+		    do_statusbar_home();
+		    break;
+		case NANO_END_KEY:
+		    do_statusbar_end();
+		    break;
+#ifndef NANO_TINY
+		case NANO_BRACKET_KEY:
+		    if (*meta_key == TRUE)
+			do_statusbar_find_bracket();
+		    break;
 #endif
 		case NANO_VERBATIM_KEY:
 		    if (*meta_key == TRUE) {
@@ -225,12 +231,22 @@ int do_statusbar_input(bool *meta_key, bool *func_key, bool *s_or_t,
 			}
 		    }
 		    break;
-#ifndef NANO_TINY
-		case NANO_BRACKET_KEY:
-		    if (*meta_key == TRUE)
-			do_statusbar_find_bracket();
+		case NANO_DELETE_KEY:
+		    /* If we're using restricted mode, the filename
+		     * isn't blank, and we're at the "Write File"
+		     * prompt, disable Delete. */
+		    if (!ISSET(RESTRICTED) || openfile->filename[0] ==
+			'\0' || currshortcut != writefile_list)
+			do_statusbar_delete();
+		    break;
+		case NANO_BACKSPACE_KEY:
+		    /* If we're using restricted mode, the filename
+		     * isn't blank, and we're at the "Write File"
+		     * prompt, disable Backspace. */
+		    if (!ISSET(RESTRICTED) || openfile->filename[0] ==
+			'\0' || currshortcut != writefile_list)
+			do_statusbar_backspace();
 		    break;
-#endif
 		/* Handle the normal statusbar prompt shortcuts, setting
 		 * ran_func to TRUE if we try to run their associated
 		 * functions and setting finished to TRUE to indicate
@@ -292,7 +308,8 @@ bool do_statusbar_mouse(void)
 
 /* The user typed output_len multibyte characters.  Add them to the
  * statusbar prompt, setting got_enter to TRUE if we get a newline, and
- * filtering out all control characters if allow_cntrls is TRUE. */
+ * filtering out all ASCII control characters if allow_cntrls is
+ * TRUE. */
 void do_statusbar_output(char *output, size_t output_len, bool
 	*got_enter, bool allow_cntrls)
 {
@@ -307,7 +324,7 @@ void do_statusbar_output(char *output, size_t output_len, bool
 
     while (i < output_len) {
 	/* If allow_cntrls is FALSE, filter out nulls and newlines,
-	 * since they're control characters. */
+	 * since they're ASCII control characters. */
 	if (allow_cntrls) {
 	    /* Null to newline, if needed. */
 	    if (output[i] == '\0')
@@ -329,8 +346,10 @@ void do_statusbar_output(char *output, size_t output_len, bool
 
 	i += char_buf_len;
 
-	/* If allow_cntrls is FALSE, filter out a control character. */
-	if (!allow_cntrls && is_cntrl_mbchar(output + i - char_buf_len))
+	/* If allow_cntrls is FALSE, filter out an ASCII control
+	 * character. */
+	if (!allow_cntrls && is_ascii_cntrl_char(*(output + i -
+		char_buf_len)))
 	    continue;
 
 	/* More dangerousness fun =) */
@@ -843,10 +862,19 @@ size_t get_statusbar_page_start(size_t start_col, size_t column)
 		start_col - 1);
 }
 
+/* Put the cursor in the statusbar prompt at statusbar_x. */
+void reset_statusbar_cursor(void)
+{
+    size_t start_col = strlenpt(prompt) + 1;
+    size_t xpt = statusbar_xplustabs();
+
+    wmove(bottomwin, 0, start_col + 1 + xpt -
+	get_statusbar_page_start(start_col, start_col + xpt));
+}
+
 /* Repaint the statusbar when getting a character in
  * get_prompt_string().  The statusbar text line will be displayed
- * starting with curranswer[index].  Assume the A_REVERSE attribute is
- * turned off. */
+ * starting with curranswer[index]. */
 void update_statusbar_line(const char *curranswer, size_t index)
 {
     size_t start_col, page_start;
@@ -858,7 +886,7 @@ void update_statusbar_line(const char *curranswer, size_t index)
     index = strnlenpt(curranswer, index);
     page_start = get_statusbar_page_start(start_col, start_col + index);
 
-    wattron(bottomwin, A_REVERSE);
+    wattron(bottomwin, reverse_attr);
 
     blank_statusbar();
 
@@ -873,17 +901,7 @@ void update_statusbar_line(const char *curranswer, size_t index)
 
     reset_statusbar_cursor();
 
-    wattroff(bottomwin, A_REVERSE);
-}
-
-/* Put the cursor in the statusbar prompt at statusbar_x. */
-void reset_statusbar_cursor(void)
-{
-    size_t start_col = strlenpt(prompt) + 1;
-    size_t xpt = statusbar_xplustabs();
-
-    wmove(bottomwin, 0, start_col + 1 + xpt -
-	get_statusbar_page_start(start_col, start_col + xpt));
+    wattroff(bottomwin, reverse_attr);
 }
 
 /* Return TRUE if we need an update after moving horizontally, and FALSE
@@ -897,6 +915,14 @@ bool need_statusbar_horizontal_update(size_t old_pww)
 	get_statusbar_page_start(start_col, start_col + statusbar_pww);
 }
 
+/* Unconditionally redraw the entire screen, and then refresh it using
+ * refresh_func(). */
+void total_statusbar_refresh(void (*refresh_func)(void))
+{
+    total_redraw();
+    refresh_func();
+}
+
 /* Get a string of input at the statusbar prompt.  This should only be
  * called from do_prompt(). */
 int get_prompt_string(bool allow_tabs,
@@ -1048,20 +1074,27 @@ int get_prompt_string(bool allow_tabs,
 		    }
 
 		    update_statusbar_line(answer, statusbar_x);
+
+		    /* This key has a shortcut list entry when it's used
+		     * to move to a newer search, which means that
+		     * finished has been set to TRUE.  Set it back to
+		     * FALSE here, so that we aren't kicked out of the
+		     * statusbar prompt. */
+		    finished = FALSE;
 		}
 		break;
 #endif /* !NANO_TINY */
-#ifndef DISABLE_HELP
 	    case NANO_HELP_KEY:
 		update_statusbar_line(answer, statusbar_x);
 
 		/* This key has a shortcut list entry when it's used to
-		 * to go to the help browser, which means that finished
-		 * has been set to TRUE.  Set it back to FALSE here, so
-		 * that we aren't kicked out of the statusbar prompt. */
+		 * go to the help browser or display a message
+		 * indicating that help is disabled, which means that
+		 * finished has been set to TRUE.  Set it back to FALSE
+		 * here, so that we aren't kicked out of the statusbar
+		 * prompt. */
 		finished = FALSE;
 		break;
-#endif /* !DISABLE_HELP */
 	}
 
 	/* If we have a shortcut with an associated function, break out
@@ -1212,9 +1245,10 @@ int do_yesno_prompt(bool all, const char *msg)
     /* yesstr, nostr, and allstr are strings of any length.  Each string
      * consists of all single-byte characters accepted as valid
      * characters for that value.  The first value will be the one
-     * displayed in the shortcuts.  Translators: if possible, specify
-     * both the shortcuts for your language and English.  For example,
-     * in French: "OoYy" for "Oui". */
+     * displayed in the shortcuts. */
+    /* TRANSLATORS: For the next three strings, if possible, specify
+     * the single-byte shortcuts for both your language and English.
+     * For example, in French: "OoYy" for "Oui". */
     yesstr = _("Yy");
     nostr = _("Nn");
     allstr = _("Aa");
@@ -1247,12 +1281,12 @@ int do_yesno_prompt(bool all, const char *msg)
 	onekey("^C", _("Cancel"), width);
     }
 
-    wattron(bottomwin, A_REVERSE);
+    wattron(bottomwin, reverse_attr);
 
     blank_statusbar();
     mvwaddnstr(bottomwin, 0, 0, msg, actual_x(msg, COLS - 1));
 
-    wattroff(bottomwin, A_REVERSE);
+    wattroff(bottomwin, reverse_attr);
 
     /* Refresh the edit window and the statusbar before getting
      * input. */
@@ -1268,24 +1302,23 @@ int do_yesno_prompt(bool all, const char *msg)
 
 	kbinput = get_kbinput(bottomwin, &meta_key, &func_key);
 
-	if (kbinput == NANO_REFRESH_KEY) {
-	    total_redraw();
-	    continue;
-	} else if (kbinput == NANO_CANCEL_KEY)
-	    ok = -1;
+	switch (kbinput) {
+	    case NANO_CANCEL_KEY:
+		ok = -1;
+		break;
 #ifndef DISABLE_MOUSE
-	else if (kbinput == KEY_MOUSE) {
-	    get_mouseinput(&mouse_x, &mouse_y, FALSE);
+	    case KEY_MOUSE:
+		get_mouseinput(&mouse_x, &mouse_y, FALSE);
 
-	    if (mouse_x != -1 && mouse_y != -1 && !ISSET(NO_HELP) &&
-		wenclose(bottomwin, mouse_y, mouse_x) &&
-		mouse_x < (width * 2) && mouse_y - (2 -
-		no_more_space()) - editwinrows - 1 >= 0) {
-		int x = mouse_x / width;
+		if (mouse_x != -1 && mouse_y != -1 && !ISSET(NO_HELP) &&
+			wenclose(bottomwin, mouse_y, mouse_x) &&
+			mouse_x < (width * 2) && mouse_y - (2 -
+			no_more_space()) - editwinrows - 1 >= 0) {
+		    int x = mouse_x / width;
 			/* Calculate the x-coordinate relative to the
 			 * two columns of the Yes/No/All shortcuts in
 			 * bottomwin. */
-		int y = mouse_y - (2 - no_more_space()) -
+		    int y = mouse_y - (2 - no_more_space()) -
 			editwinrows - 1;
 			/* Calculate the y-coordinate relative to the
 			 * beginning of the Yes/No/All shortcuts in
@@ -1293,25 +1326,30 @@ int do_yesno_prompt(bool all, const char *msg)
 			 * edit, and the first line of bottomwin
 			 * subtracted out. */
 
-		assert(0 <= x && x <= 1 && 0 <= y && y <= 1);
+		    assert(0 <= x && x <= 1 && 0 <= y && y <= 1);
 
-		/* x == 0 means they clicked Yes or No.  y == 0 means
-		 * Yes or All. */
-		ok = -2 * x * y + x - y + 1;
+		    /* x == 0 means they clicked Yes or No.  y == 0
+		     * means Yes or All. */
+		    ok = -2 * x * y + x - y + 1;
 
-		if (ok == 2 && !all)
-		    ok = -2;
-	    }
+		    if (ok == 2 && !all)
+			ok = -2;
+		}
+		break;
+#endif /* !DISABLE_MOUSE */
+	    case NANO_REFRESH_KEY:
+		total_redraw();
+		continue;
+	    default:
+		/* Look for the kbinput in the Yes, No and (optionally)
+		 * All strings. */
+		if (strchr(yesstr, kbinput) != NULL)
+		    ok = 1;
+		else if (strchr(nostr, kbinput) != NULL)
+		    ok = 0;
+		else if (all && strchr(allstr, kbinput) != NULL)
+		    ok = 2;
 	}
-#endif
-	/* Look for the kbinput in the Yes, No and (optionally) All
-	 * strings. */
-	else if (strchr(yesstr, kbinput) != NULL)
-	    ok = 1;
-	else if (strchr(nostr, kbinput) != NULL)
-	    ok = 0;
-	else if (all && strchr(allstr, kbinput) != NULL)
-	    ok = 2;
     } while (ok == -2);
 
     return ok;
