@@ -1,4 +1,4 @@
-/* $Id: proto.h,v 1.326 2006/02/18 21:32:29 dolorous Exp $ */
+/* $Id: proto.h,v 1.350 2006/06/21 16:54:51 dolorous Exp $ */
 /**************************************************************************
  *   proto.h                                                              *
  *                                                                        *
@@ -27,6 +27,11 @@
 #include "nano.h"
 
 /* Public externs.  See global.c for descriptions of them. */
+#ifndef NANO_TINY
+extern sigjmp_buf jump_buf;
+extern bool jump_buf_main;
+#endif
+
 #ifndef DISABLE_WRAPJUSTIFY
 extern ssize_t fill;
 extern ssize_t wrap_at;
@@ -104,6 +109,7 @@ extern shortcut *spell_list;
 #endif
 #ifndef DISABLE_BROWSER
 extern shortcut *browser_list;
+extern shortcut *whereis_file_list;
 extern shortcut *gotodir_list;
 #endif
 
@@ -131,7 +137,7 @@ extern regex_t search_regexp;
 extern regmatch_t regmatches[10];
 #endif
 
-extern bool curses_ended;
+extern int reverse_attr;
 
 extern char *homedir;
 
@@ -142,10 +148,22 @@ char *do_browse_from(const char *inpath);
 void browser_init(const char *path, DIR *dir);
 void parse_browser_input(int *kbinput, bool *meta_key, bool *func_key);
 void browser_refresh(void);
+int filesearch_init(void);
+bool findnextfile(bool no_sameline, size_t begin, const char *needle);
+void findnextfile_wrap_reset(void);
+void filesearch_abort(void);
+void do_filesearch(void);
+void do_fileresearch(void);
+void do_first_file(void);
+void do_last_file(void);
 void striponedir(char *path);
 #endif
 
 /* Public functions in chars.c. */
+#ifdef ENABLE_UTF8
+void utf8_init(void);
+bool using_utf8(void);
+#endif
 #ifndef HAVE_ISBLANK
 bool nisblank(int c);
 #endif
@@ -155,6 +173,7 @@ bool niswblank(wchar_t wc);
 bool is_byte(int c);
 bool is_alnum_mbchar(const char *c);
 bool is_blank_mbchar(const char *c);
+bool is_ascii_cntrl_char(int c);
 bool is_cntrl_char(int c);
 #ifdef ENABLE_UTF8
 bool is_cntrl_wchar(wchar_t wc);
@@ -235,9 +254,18 @@ void cut_line(void);
 #ifndef NANO_TINY
 void cut_marked(void);
 void cut_to_eol(void);
+void cut_to_eof(void);
 #endif
-void do_cut_text(void);
+void do_cut_text(
 #ifndef NANO_TINY
+	bool copy_text, bool cut_till_end
+#else
+	void
+#endif
+	);
+void do_cut_text_void(void);
+#ifndef NANO_TINY
+void do_copy_text(void);
 void do_cut_till_end(void);
 #endif
 void do_uncut_text(void);
@@ -313,15 +341,16 @@ void save_history(void);
 /* Public functions in global.c. */
 size_t length_of_list(const shortcut *s);
 #ifndef NANO_TINY
-void toggle_init_one(int val, const char *desc, long flag);
+void toggle_init_one(int val, const char *desc, bool blank_after, long
+	flag);
 void toggle_init(void);
 #endif
-void sc_init_one(shortcut **shortcutage, int key, const char *desc,
+void sc_init_one(shortcut **shortcutage, int ctrlval, const char *desc,
 #ifndef DISABLE_HELP
 	const char *help,
 #endif
-	int metaval, int funcval, int miscval, bool view, void
-	(*func)(void));
+	bool blank_after, int metaval, int funcval, int miscval, bool
+	view, void (*func)(void));
 void shortcut_init(bool unjustify);
 void free_shortcutage(shortcut **shortcutage);
 #ifdef DEBUG
@@ -330,8 +359,13 @@ void thanks_for_all_the_fish(void);
 
 /* Public functions in help.c. */
 #ifndef DISABLE_HELP
-void do_help(void);
+void do_help(void (*refresh_func)(void));
+void do_help_void(void);
+#ifndef DISABLE_BROWSER
+void do_browser_help(void);
+#endif
 void help_init(void);
+void parse_help_input(int *kbinput, bool *meta_key, bool *func_key);
 size_t help_line_len(const char *ptr);
 #endif
 
@@ -395,9 +429,11 @@ void die(const char *msg, ...);
 void die_save_file(const char *die_filename);
 void window_init(void);
 #ifndef DISABLE_MOUSE
+void disable_mouse_support(void);
+void enable_mouse_support(void);
 void mouse_init(void);
 #endif
-void print1opt_full(const char *shortflag
+void print_opt_full(const char *shortflag
 #ifdef HAVE_GETOPT_LONG
 	, const char *longflag
 #endif
@@ -462,9 +498,10 @@ void do_statusbar_find_bracket(void);
 #endif
 size_t statusbar_xplustabs(void);
 size_t get_statusbar_page_start(size_t start_col, size_t column);
-void update_statusbar_line(const char *curranswer, size_t index);
 void reset_statusbar_cursor(void);
+void update_statusbar_line(const char *curranswer, size_t index);
 bool need_statusbar_horizontal_update(size_t old_pww);
+void total_statusbar_refresh(void (*refresh_func)(void));
 int get_prompt_string(bool allow_tabs,
 #ifndef DISABLE_TABCOMP
 	bool allow_files,
@@ -496,13 +533,18 @@ void rcfile_error(const char *msg, ...);
 char *parse_next_word(char *ptr);
 char *parse_argument(char *ptr);
 #ifdef ENABLE_COLOR
-short color_to_short(const char *colorname, bool *bright);
 char *parse_next_regex(char *ptr);
 bool nregcomp(const char *regex, int eflags);
 void parse_syntax(char *ptr);
+void parse_include(char *ptr);
+short color_to_short(const char *colorname, bool *bright);
 void parse_colors(char *ptr, bool icase);
 #endif
-void parse_rcfile(FILE *rcstream);
+void parse_rcfile(FILE *rcstream
+#ifdef ENABLE_COLOR
+	, bool syntax_only
+#endif
+	);
 void do_rcfile(void);
 #endif
 
@@ -569,6 +611,11 @@ void do_mark(void);
 void do_delete(void);
 void do_backspace(void);
 void do_tab(void);
+#ifndef NANO_TINY
+void do_indent_marked(ssize_t cols);
+void do_indent_marked_void(void);
+void do_unindent_marked_void(void);
+#endif
 void do_enter(void);
 #ifndef NANO_TINY
 RETSIGTYPE cancel_command(int signal);
@@ -632,10 +679,6 @@ ssize_t ngetdelim(char **lineptr, size_t *n, int delim, FILE *stream);
 #endif
 #endif
 #ifdef HAVE_REGEX_H
-#ifdef BROKEN_REGEXEC
-int safe_regexec(const regex_t *preg, const char *string, size_t nmatch,
-	regmatch_t pmatch[], int eflags);
-#endif
 bool regexp_bol_or_eol(const regex_t *preg, const char *string);
 #endif
 #ifndef DISABLE_SPELLER
@@ -661,42 +704,28 @@ void mark_order(const filestruct **top, size_t *top_x, const filestruct
 	**bot, size_t *bot_x, bool *right_side_up);
 #endif
 size_t get_totsize(const filestruct *begin, const filestruct *end);
-#ifndef NDEBUG
-int check_linenumbers(const filestruct *fileptr);
-#endif
 #ifdef DEBUG
 void dump_filestruct(const filestruct *inptr);
 void dump_filestruct_reverse(void);
 #endif
 
 /* Public functions in winio.c. */
-#ifndef NANO_TINY
-void reset_kbinput(void);
-#endif
 void get_key_buffer(WINDOW *win);
 size_t get_key_buffer_len(void);
 void unget_input(int *input, size_t input_len);
 void unget_kbinput(int kbinput, bool meta_key, bool func_key);
 int *get_input(WINDOW *win, size_t input_len);
 int get_kbinput(WINDOW *win, bool *meta_key, bool *func_key);
-int parse_kbinput(WINDOW *win, bool *meta_key, bool *func_key
-#ifndef NANO_TINY
-	, bool reset
-#endif
-	);
+int parse_kbinput(WINDOW *win, bool *meta_key, bool *func_key);
 int get_escape_seq_kbinput(const int *seq, size_t seq_len, bool
 	*ignore_seq);
 int get_escape_seq_abcd(int kbinput);
-int get_byte_kbinput(int kbinput
-#ifndef NANO_TINY
-	, bool reset
+int parse_escape_seq_kbinput(int kbinput, bool *ignore_seq);
+int get_byte_kbinput(int kbinput);
+#ifdef ENABLE_UTF8
+long add_unicode_digit(int kbinput, long factor, long *uni);
+long get_unicode_kbinput(int kbinput);
 #endif
-	);
-long get_unicode_kbinput(int kbinput
-#ifndef NANO_TINY
-	, bool reset
-#endif
-	);
 int get_control_kbinput(int kbinput);
 void unparse_kbinput(char *output, size_t output_len);
 int *get_verbatim_kbinput(WINDOW *win, size_t *kbinput_len);
