@@ -1,4 +1,4 @@
-/* $Id: rcfile.c,v 1.93 2006/01/06 22:04:38 dolorous Exp $ */
+/* $Id: rcfile.c,v 1.115 2006/06/08 02:50:56 dolorous Exp $ */
 /**************************************************************************
  *   rcfile.c                                                             *
  *                                                                        *
@@ -33,6 +33,7 @@
 #ifdef ENABLE_NANORC
 
 const static rcoption rcopts[] = {
+    {"boldtext", BOLD_TEXT},
 #ifndef DISABLE_JUSTIFY
     {"brackets", 0},
 #endif
@@ -108,7 +109,7 @@ static colortype *endcolor = NULL;
 #endif
 
 /* We have an error in some part of the rcfile.  Put it on stderr and
- * make the user hit Enter to continue starting up nano. */
+ * make the user hit Enter to continue starting nano. */
 void rcfile_error(const char *msg, ...)
 {
     va_list ap;
@@ -151,7 +152,7 @@ char *parse_next_word(char *ptr)
  * arguments can contain "'s too. */
 char *parse_argument(char *ptr)
 {
-    const char *ptr_bak = ptr;
+    const char *ptr_save = ptr;
     char *last_quote = NULL;
 
     assert(ptr != NULL);
@@ -170,7 +171,7 @@ char *parse_argument(char *ptr)
 	    ptr = NULL;
 	else
 	    *ptr++ = '\0';
-	rcfile_error(N_("Argument %s has unterminated \""), ptr_bak);
+	rcfile_error(N_("Argument '%s' has an unterminated \""), ptr_save);
     } else {
 	*last_quote = '\0';
 	ptr = last_quote + 1;
@@ -182,45 +183,6 @@ char *parse_argument(char *ptr)
 }
 
 #ifdef ENABLE_COLOR
-/* Return the short value corresponding to the color named in colorname,
- * and set bright to TRUE if that color is bright. */
-short color_to_short(const char *colorname, bool *bright)
-{
-    short mcolor = -1;
-
-    assert(colorname != NULL && bright != NULL);
-
-    if (strncasecmp(colorname, "bright", 6) == 0) {
-	*bright = TRUE;
-	colorname += 6;
-    }
-
-    if (strcasecmp(colorname, "green") == 0)
-	mcolor = COLOR_GREEN;
-    else if (strcasecmp(colorname, "red") == 0)
-	mcolor = COLOR_RED;
-    else if (strcasecmp(colorname, "blue") == 0)
-	mcolor = COLOR_BLUE;
-    else if (strcasecmp(colorname, "white") == 0)
-	mcolor = COLOR_WHITE;
-    else if (strcasecmp(colorname, "yellow") == 0)
-	mcolor = COLOR_YELLOW;
-    else if (strcasecmp(colorname, "cyan") == 0)
-	mcolor = COLOR_CYAN;
-    else if (strcasecmp(colorname, "magenta") == 0)
-	mcolor = COLOR_MAGENTA;
-    else if (strcasecmp(colorname, "black") == 0)
-	mcolor = COLOR_BLACK;
-    else
-	rcfile_error(N_("Color %s not understood.\n"
-		"Valid colors are \"green\", \"red\", \"blue\",\n"
-		"\"white\", \"yellow\", \"cyan\", \"magenta\" and\n"
-		"\"black\", with the optional prefix \"bright\"\n"
-		"for foreground colors."), colorname);
-
-    return mcolor;
-}
-
 /* Parse the next regex string from the line at ptr, and return it. */
 char *parse_next_regex(char *ptr)
 {
@@ -275,7 +237,7 @@ bool nregcomp(const char *regex, int eflags)
 void parse_syntax(char *ptr)
 {
     const char *fileregptr = NULL, *nameptr = NULL;
-    const syntaxtype *tmpsyntax;
+    syntaxtype *tmpsyntax;
     exttype *endext = NULL;
 	/* The end of the extensions list for this syntax. */
 
@@ -300,11 +262,16 @@ void parse_syntax(char *ptr)
     if (ptr == NULL)
 	return;
 
+    /* Search for a duplicate syntax name.  If we find one, free it, so
+     * that we always use the last syntax with a given name. */
     for (tmpsyntax = syntaxes; tmpsyntax != NULL;
 	tmpsyntax = tmpsyntax->next) {
 	if (strcmp(nameptr, tmpsyntax->desc) == 0) {
-	    rcfile_error(N_("Duplicate syntax name %s"), nameptr);
-	    return;
+	    syntaxtype *prev_syntax = tmpsyntax;
+
+	    tmpsyntax = tmpsyntax->next;
+	    free(prev_syntax);
+	    break;
 	}
     }
 
@@ -379,6 +346,104 @@ void parse_syntax(char *ptr)
     }
 }
 
+/* Read and parse additional syntax files. */
+void parse_include(char *ptr)
+{
+    struct stat rcinfo;
+    FILE *rcstream;
+    char *option, *full_option, *nanorc_save = nanorc;
+    size_t lineno_save = lineno;
+
+    option = ptr;
+    if (*option == '"')
+	option++;
+    ptr = parse_argument(ptr);
+
+    /* Get the specified file's full path. */
+    full_option = get_full_path(option);
+
+    if (full_option == NULL) {
+	rcfile_error(_("Error reading %s: %s"), option, strerror(errno));
+	goto cleanup_include;
+    }
+
+    /* Don't open directories, character files, or block files. */
+    if (stat(nanorc, &rcinfo) != -1) {
+	if (S_ISDIR(rcinfo.st_mode) || S_ISCHR(rcinfo.st_mode) ||
+		S_ISBLK(rcinfo.st_mode)) {
+	    rcfile_error(S_ISDIR(rcinfo.st_mode) ?
+		_("\"%s\" is a directory") :
+		_("\"%s\" is a device file"), nanorc);
+	    goto cleanup_include;
+	}
+    }
+
+    /* Open the new syntax file. */
+    if ((rcstream = fopen(full_option, "rb")) == NULL) {
+	rcfile_error(_("Error reading %s: %s"), full_option,
+		strerror(errno));
+	goto cleanup_include;
+    }
+
+    /* Use the name and line number position of the new syntax file
+     * while parsing it, so we can know where any errors in it are. */
+    nanorc = full_option;
+    lineno = 0;
+
+    parse_rcfile(rcstream
+#ifdef ENABLE_COLOR
+	, TRUE
+#endif
+	);
+
+    /* We're done with the new syntax file.  Restore the original
+     * filename and line number position. */
+    nanorc = nanorc_save;
+    lineno = lineno_save;
+
+  cleanup_include:
+    free(full_option);
+}
+
+/* Return the short value corresponding to the color named in colorname,
+ * and set bright to TRUE if that color is bright. */
+short color_to_short(const char *colorname, bool *bright)
+{
+    short mcolor = -1;
+
+    assert(colorname != NULL && bright != NULL);
+
+    if (strncasecmp(colorname, "bright", 6) == 0) {
+	*bright = TRUE;
+	colorname += 6;
+    }
+
+    if (strcasecmp(colorname, "green") == 0)
+	mcolor = COLOR_GREEN;
+    else if (strcasecmp(colorname, "red") == 0)
+	mcolor = COLOR_RED;
+    else if (strcasecmp(colorname, "blue") == 0)
+	mcolor = COLOR_BLUE;
+    else if (strcasecmp(colorname, "white") == 0)
+	mcolor = COLOR_WHITE;
+    else if (strcasecmp(colorname, "yellow") == 0)
+	mcolor = COLOR_YELLOW;
+    else if (strcasecmp(colorname, "cyan") == 0)
+	mcolor = COLOR_CYAN;
+    else if (strcasecmp(colorname, "magenta") == 0)
+	mcolor = COLOR_MAGENTA;
+    else if (strcasecmp(colorname, "black") == 0)
+	mcolor = COLOR_BLACK;
+    else
+	rcfile_error(N_("Color \"%s\" not understood.\n"
+		"Valid colors are \"green\", \"red\", \"blue\",\n"
+		"\"white\", \"yellow\", \"cyan\", \"magenta\" and\n"
+		"\"black\", with the optional prefix \"bright\"\n"
+		"for foreground colors."), colorname);
+
+    return mcolor;
+}
+
 /* Parse the color string in the line at ptr, and add it to the current
  * file's associated colors.  If icase is TRUE, treat the color string
  * as case insensitive. */
@@ -390,6 +455,12 @@ void parse_colors(char *ptr, bool icase)
 
     assert(ptr != NULL);
 
+    if (syntaxes == NULL) {
+	rcfile_error(
+		N_("Cannot add a color command without a syntax command"));
+	return;
+    }
+
     if (*ptr == '\0') {
 	rcfile_error(N_("Missing color name"));
 	return;
@@ -411,7 +482,7 @@ void parse_colors(char *ptr, bool icase)
 	}
 	if (strncasecmp(bgcolorname, "bright", 6) == 0) {
 	    rcfile_error(
-		N_("Background color %s cannot be bright"),
+		N_("Background color \"%s\" cannot be bright"),
 		bgcolorname);
 	    return;
 	}
@@ -428,12 +499,6 @@ void parse_colors(char *ptr, bool icase)
     } else
 	fg = -1;
 
-    if (syntaxes == NULL) {
-	rcfile_error(
-		N_("Cannot add a color directive without a syntax line"));
-	return;
-    }
-
     if (*ptr == '\0') {
 	rcfile_error(N_("Missing regex string"));
 	return;
@@ -537,9 +602,14 @@ void parse_colors(char *ptr, bool icase)
 }
 #endif /* ENABLE_COLOR */
 
-/* Parse the rcfile, once it has been opened successfully at
- * rcstream. */
-void parse_rcfile(FILE *rcstream)
+/* Parse the rcfile, once it has been opened successfully at rcstream,
+ * and close it afterwards.  If syntax_only is TRUE, only allow the file
+ * to contain color syntax commands: syntax, color, and icolor. */
+void parse_rcfile(FILE *rcstream
+#ifdef ENABLE_COLOR
+	, bool syntax_only
+#endif
+	)
 {
     char *buf = NULL;
     ssize_t len;
@@ -547,10 +617,12 @@ void parse_rcfile(FILE *rcstream)
 
     while ((len = getline(&buf, &n, rcstream)) > 0) {
 	char *ptr, *keyword, *option;
-	int set = 0, i;
+	int set = 0;
+	size_t i;
 
-	/* Ignore the \n. */
-	buf[len - 1] = '\0';
+	/* Ignore the newline. */
+	if (buf[len - 1] == '\n')
+	    buf[len - 1] = '\0';
 
 	lineno++;
 	ptr = buf;
@@ -567,20 +639,45 @@ void parse_rcfile(FILE *rcstream)
 	ptr = parse_next_word(ptr);
 
 	/* Try to parse the keyword. */
-	if (strcasecmp(keyword, "set") == 0)
-	    set = 1;
-	else if (strcasecmp(keyword, "unset") == 0)
-	    set = -1;
+	if (strcasecmp(keyword, "set") == 0) {
 #ifdef ENABLE_COLOR
-	else if (strcasecmp(keyword, "syntax") == 0)
+	    if (syntax_only)
+		rcfile_error(
+			N_("Command \"%s\" not allowed in included file"),
+			keyword);
+	    else
+#endif
+		set = 1;
+	} else if (strcasecmp(keyword, "unset") == 0) {
+#ifdef ENABLE_COLOR
+	    if (syntax_only)
+		rcfile_error(
+			N_("Command \"%s\" not allowed in included file"),
+			keyword);
+	    else
+#endif
+		set = -1;
+	}
+#ifdef ENABLE_COLOR
+	else if (strcasecmp(keyword, "include") == 0) {
+	    if (syntax_only)
+		rcfile_error(
+			N_("Command \"%s\" not allowed in included file"),
+			keyword);
+	    else
+		parse_include(ptr);
+	} else if (strcasecmp(keyword, "syntax") == 0) {
+	    if (endsyntax != NULL && endcolor == NULL)
+		rcfile_error(N_("Syntax \"%s\" has no color commands"),
+			endsyntax->desc);
 	    parse_syntax(ptr);
-	else if (strcasecmp(keyword, "color") == 0)
+	} else if (strcasecmp(keyword, "color") == 0)
 	    parse_colors(ptr, FALSE);
 	else if (strcasecmp(keyword, "icolor") == 0)
 	    parse_colors(ptr, TRUE);
 #endif /* ENABLE_COLOR */
 	else
-	    rcfile_error(N_("Command %s not understood"), keyword);
+	    rcfile_error(N_("Command \"%s\" not understood"), keyword);
 
 	if (set == 0)
 	    continue;
@@ -605,10 +702,10 @@ void parse_rcfile(FILE *rcstream)
 			SET(rcopts[i].flag);
 		    else {
 			/* This option doesn't have a flag, so it takes
-			 *an argument. */
+			 * an argument. */
 			if (*ptr == '\0') {
 			    rcfile_error(
-				N_("Option %s requires an argument"),
+				N_("Option \"%s\" requires an argument"),
 				rcopts[i].name);
 			    break;
 			}
@@ -639,7 +736,7 @@ void parse_rcfile(FILE *rcstream)
 			if (strcasecmp(rcopts[i].name, "fill") == 0) {
 			    if (!parse_num(option, &wrap_at)) {
 				rcfile_error(
-					N_("Requested fill size %s invalid"),
+					N_("Requested fill size \"%s\" is invalid"),
 					option);
 				wrap_at = -CHARS_FROM_EOL;
 			    } else
@@ -714,7 +811,7 @@ void parse_rcfile(FILE *rcstream)
 			    if (!parse_num(option, &tabsize) ||
 				tabsize <= 0) {
 				rcfile_error(
-					N_("Requested tab size %s invalid"),
+					N_("Requested tab size \"%s\" is invalid"),
 					option);
 				tabsize = -1;
 			    } else
@@ -728,22 +825,27 @@ void parse_rcfile(FILE *rcstream)
 		} else if (rcopts[i].flag != 0)
 		    UNSET(rcopts[i].flag);
 		else
-		    rcfile_error(N_("Cannot unset flag %s"),
+		    rcfile_error(N_("Cannot unset flag \"%s\""),
 			rcopts[i].name);
 		break;
 	    }
 	}
 	if (rcopts[i].name == NULL)
-	    rcfile_error(N_("Unknown flag %s"), option);
+	    rcfile_error(N_("Unknown flag \"%s\""), option);
     }
 
+    if (endsyntax != NULL && endcolor == NULL)
+	rcfile_error(N_("Syntax \"%s\" has no color commands"),
+		endsyntax->desc);
+
     free(buf);
     fclose(rcstream);
     lineno = 0;
 
     if (errors) {
 	errors = FALSE;
-	fprintf(stderr, _("\nPress Enter to continue starting nano\n"));
+	fprintf(stderr,
+		_("\nPress Enter to continue starting nano.\n"));
 	while (getchar() != '\n')
 	    ;
     }
@@ -752,18 +854,31 @@ void parse_rcfile(FILE *rcstream)
 }
 
 /* The main rcfile function.  It tries to open the system-wide rcfile,
- * followed by the local rcfile. */
+ * followed by the current user's rcfile. */
 void do_rcfile(void)
 {
+    struct stat rcinfo;
     FILE *rcstream;
 
-#ifdef SYSCONFDIR
     nanorc = mallocstrcpy(nanorc, SYSCONFDIR "/nanorc");
+
+    /* Don't open directories, character files, or block files. */
+    if (stat(nanorc, &rcinfo) != -1) {
+	if (S_ISDIR(rcinfo.st_mode) || S_ISCHR(rcinfo.st_mode) ||
+		S_ISBLK(rcinfo.st_mode))
+	    rcfile_error(S_ISDIR(rcinfo.st_mode) ?
+		_("\"%s\" is a directory") :
+		_("\"%s\" is a device file"), nanorc);
+    }
+
     /* Try to open the system-wide nanorc. */
-    rcstream = fopen(nanorc, "r");
+    rcstream = fopen(nanorc, "rb");
     if (rcstream != NULL)
-	parse_rcfile(rcstream);
+	parse_rcfile(rcstream
+#ifdef ENABLE_COLOR
+		, FALSE
 #endif
+		);
 
 #if defined(DISABLE_ROOTWRAP) && !defined(DISABLE_WRAPPING)
     /* We've already read SYSCONFDIR/nanorc, if it's there.  If we're
@@ -780,15 +895,29 @@ void do_rcfile(void)
     else {
 	nanorc = charealloc(nanorc, strlen(homedir) + 9);
 	sprintf(nanorc, "%s/.nanorc", homedir);
-	rcstream = fopen(nanorc, "r");
 
+	/* Don't open directories, character files, or block files. */
+	if (stat(nanorc, &rcinfo) != -1) {
+	    if (S_ISDIR(rcinfo.st_mode) || S_ISCHR(rcinfo.st_mode) ||
+		S_ISBLK(rcinfo.st_mode))
+		rcfile_error(S_ISDIR(rcinfo.st_mode) ?
+			_("\"%s\" is a directory") :
+			_("\"%s\" is a device file"), nanorc);
+	}
+
+	/* Try to open the current user's nanorc. */
+	rcstream = fopen(nanorc, "rb");
 	if (rcstream == NULL) {
 	    /* Don't complain about the file's not existing. */
 	    if (errno != ENOENT)
 		rcfile_error(N_("Error reading %s: %s"), nanorc,
 			strerror(errno));
 	} else
-	    parse_rcfile(rcstream);
+	    parse_rcfile(rcstream
+#ifdef ENABLE_COLOR
+		, FALSE
+#endif
+		);
     }
 
     free(nanorc);
