@@ -1,4 +1,4 @@
-/* $Id: search.c,v 1.187 2006/03/24 04:22:45 dolorous Exp $ */
+/* $Id: search.c,v 1.189 2006/05/22 02:08:49 dolorous Exp $ */
 /**************************************************************************
  *   search.c                                                             *
  *                                                                        *
@@ -178,20 +178,20 @@ int search_init(bool replacing, bool use_answer)
 #endif
 	edit_refresh, "%s%s%s%s%s%s", _("Search"),
 #ifndef NANO_TINY
-	/* This string is just a modifier for the search prompt; no
-	 * grammar is implied. */
+	/* TRANSLATORS: This string is just a modifier for the search
+	 * prompt; no grammar is implied. */
 	ISSET(CASE_SENSITIVE) ? _(" [Case Sensitive]") :
 #endif
 	"",
 #ifdef HAVE_REGEX_H
-	/* This string is just a modifier for the search prompt; no
-	 * grammar is implied. */
+	/* TRANSLATORS: This string is just a modifier for the search
+	 * prompt; no grammar is implied. */
 	ISSET(USE_REGEXP) ? _(" [Regexp]") :
 #endif
 	"",
 #ifndef NANO_TINY
-	/* This string is just a modifier for the search prompt; no
-	 * grammar is implied. */
+	/* TRANSLATORS: This string is just a modifier for the search
+	 * prompt; no grammar is implied. */
 	ISSET(BACKWARDS_SEARCH) ? _(" [Backwards]") :
 #endif
 	"", replacing ?
@@ -425,19 +425,18 @@ void do_search(void)
     int i;
     bool didfind;
 
-#ifndef DISABLE_WRAPPING
-    wrap_reset();
-#endif
-
     i = search_init(FALSE, FALSE);
-    if (i == -1)	/* Cancel, Go to Line, blank search string, or
-			 * regcomp() failed. */
+
+    if (i == -1)
+	/* Cancel, Go to Line, blank search string, or regcomp()
+	 * failed. */
 	search_replace_abort();
-    else if (i == -2)	/* Replace. */
+    else if (i == -2)
+	/* Replace. */
 	do_replace();
 #if !defined(NANO_TINY) || defined(HAVE_REGEX_H)
-    else if (i == 1)	/* Case Sensitive, Backwards, or Regexp search
-			 * toggle. */
+    else if (i == 1)
+	/* Case Sensitive, Backwards, or Regexp search toggle. */
 	do_search();
 #endif
 
@@ -507,10 +506,6 @@ void do_research(void)
     size_t old_pww = openfile->placewewant;
     bool didfind;
 
-#ifndef DISABLE_WRAPPING
-    wrap_reset();
-#endif
-
     search_init_globals();
 
     if (last_search[0] != '\0') {
@@ -888,15 +883,17 @@ void do_replace(void)
     }
 
     i = search_init(TRUE, FALSE);
-    if (i == -1) {		/* Cancel, Go to Line, blank search
-				 * string, or regcomp() failed. */
+    if (i == -1) {
+	/* Cancel, Go to Line, blank search string, or regcomp()
+	 * failed. */
 	search_replace_abort();
 	return;
-    } else if (i == -2) {	/* No Replace. */
+    } else if (i == -2) {
+	/* No Replace. */
 	do_search();
 	return;
-    } else if (i == 1)		/* Case Sensitive, Backwards, or Regexp
-				 * search toggle. */
+    } else if (i == 1)
+	/* Case Sensitive, Backwards, or Regexp search toggle. */
 	do_replace();
 
     if (i != 0)
