@@ -1,4 +1,4 @@
-/* $Id: text.c,v 1.112 2006/02/18 21:32:29 dolorous Exp $ */
+/* $Id: text.c,v 1.150 2006/06/11 19:15:59 dolorous Exp $ */
 /**************************************************************************
  *   text.c                                                               *
  *                                                                        *
@@ -68,7 +68,7 @@ void do_delete(void)
 {
     bool do_refresh = FALSE;
 	/* Do we have to call edit_refresh(), or can we get away with
-	 * update_line()? */
+	 * just update_line()? */
 
     assert(openfile->current != NULL && openfile->current->data != NULL && openfile->current_x <= strlen(openfile->current->data));
 
@@ -124,9 +124,6 @@ void do_delete(void)
 	delete_node(foo);
 	renumber(openfile->current);
 	openfile->totsize--;
-#ifndef DISABLE_WRAPPING
-	wrap_reset();
-#endif
 
 	/* If the NO_NEWLINES flag isn't set, and text has been added to
 	 * the magicline as a result of deleting at the end of the line
@@ -193,6 +190,176 @@ void do_tab(void)
 #endif
 }
 
+#ifndef NANO_TINY
+/* Indent or unindent all lines covered by the mark len columns,
+ * depending on whether len is positive or negative.  If the
+ * TABS_TO_SPACES flag is set, indent/unindent by len spaces.
+ * Otherwise, indent/unindent by (len / tabsize) tabs and (len %
+ * tabsize) spaces. */
+void do_indent_marked(ssize_t cols)
+{
+    bool indent_changed = FALSE;
+	/* Whether any indenting or unindenting was done. */
+    bool unindent = FALSE;
+	/* Whether we're unindenting text. */
+    char *line_indent = NULL;
+	/* The text added to each line in order to indent it. */
+    size_t line_indent_len = 0;
+	/* The length of the text added to each line in order to indent
+	 * it. */
+    filestruct *top, *bot, *f;
+    size_t top_x, bot_x;
+
+    assert(openfile->current != NULL && openfile->current->data != NULL);
+
+    /* If the mark isn't on, indicate it on the statusbar and get
+     * out. */
+    if (!openfile->mark_set) {
+	statusbar(_("No lines selected, nothing to do!"));
+	return;
+    }
+
+    /* If cols is zero, get out. */
+    if (cols == 0)
+	return;
+
+    /* If cols is negative, make it positive and set unindent to
+     * TRUE. */
+    if (cols < 0) {
+	cols = -cols;
+	unindent = TRUE;
+    /* Otherwise, we're indenting, in which case the file will always be
+     * modified, so set indent_changed to TRUE. */
+    } else
+	indent_changed = TRUE;
+
+    /* Get the coordinates of the marked text. */
+    mark_order((const filestruct **)&top, &top_x,
+	(const filestruct **)&bot, &bot_x, NULL);
+
+    if (!unindent) {
+	/* Set up the text we'll be using as indentation. */
+	line_indent = charalloc(cols + 1);
+
+	if (ISSET(TABS_TO_SPACES)) {
+	    /* Set the indentation to cols spaces. */
+	    charset(line_indent, ' ', cols);
+	    line_indent_len = cols;
+	} else {
+	    /* Set the indentation to (cols / tabsize) tabs and (cols %
+	     * tabsize) spaces. */
+	    size_t num_tabs = cols / tabsize;
+	    size_t num_spaces = cols % tabsize;
+
+	    charset(line_indent, '\t', num_tabs);
+	    charset(line_indent + num_tabs, ' ', num_spaces);
+
+	    line_indent_len = num_tabs + num_spaces;
+	}
+
+	line_indent[line_indent_len] = '\0';
+    }
+
+    /* Go through each line of the marked text. */
+    for (f = top; f != bot->next; f = f->next) {
+	size_t line_len = strlen(f->data);
+	size_t indent_len = indent_length(f->data);
+
+	if (!unindent) {
+	    /* If we're indenting, add the characters in line_indent to
+	     * the beginning of the non-whitespace text of this line. */
+	    f->data = charealloc(f->data, line_len +
+		line_indent_len + 1);
+	    charmove(&f->data[indent_len + line_indent_len],
+		&f->data[indent_len], line_len - indent_len + 1);
+	    strncpy(f->data + indent_len, line_indent, line_indent_len);
+	    openfile->totsize += line_indent_len;
+
+	    /* Keep track of the change in the current line. */
+	    if (f == openfile->mark_begin && openfile->mark_begin_x >=
+		indent_len)
+		openfile->mark_begin_x += line_indent_len;
+
+	    if (f == openfile->current && openfile->current_x >=
+		indent_len)
+		openfile->current_x += line_indent_len;
+
+	    /* If the NO_NEWLINES flag isn't set, and this is the
+	     * magicline, add a new magicline. */
+	    if (!ISSET(NO_NEWLINES) && f == openfile->filebot)
+		new_magicline();
+	} else {
+	    size_t indent_col = strnlenpt(f->data, indent_len);
+		/* The length in columns of the indentation on this
+		 * line. */
+
+	    if (cols <= indent_col) {
+		size_t indent_new = actual_x(f->data, indent_col -
+			cols);
+			/* The length of the indentation remaining on
+			 * this line after we unindent. */
+		size_t indent_shift = indent_len - indent_new;
+			/* The change in the indentation on this line
+			 * after we unindent. */
+
+		/* If we're unindenting, and there's at least cols
+		 * columns' worth of indentation at the beginning of the
+		 * non-whitespace text of this line, remove it. */
+		charmove(&f->data[indent_new], &f->data[indent_len],
+			line_len - indent_shift - indent_new + 1);
+		null_at(&f->data, line_len - indent_shift + 1);
+		openfile->totsize -= indent_shift;
+
+		/* Keep track of the change in the current line. */
+		if (f == openfile->mark_begin &&
+			openfile->mark_begin_x > indent_new) {
+		    if (openfile->mark_begin_x <= indent_len)
+			openfile->mark_begin_x = indent_new;
+		    else
+			openfile->mark_begin_x -= indent_shift;
+		}
+
+		if (f == openfile->current && openfile->current_x >
+			indent_new) {
+		    if (openfile->current_x <= indent_len)
+			openfile->current_x = indent_new;
+		    else
+			openfile->current_x -= indent_shift;
+		}
+
+		/* We've unindented, so set indent_changed to TRUE. */
+		if (!indent_changed)
+		    indent_changed = TRUE;
+	    }
+	}
+    }
+
+    if (!unindent)
+	/* Clean up. */
+	free(line_indent);
+
+    if (indent_changed) {
+	/* Mark the file as modified. */
+	set_modified();
+
+	/* Update the screen. */
+	edit_refresh();
+    }
+}
+
+/* Indent all lines covered by the mark tabsize columns. */
+void do_indent_marked_void(void)
+{
+    do_indent_marked(tabsize);
+}
+
+/* Unindent all lines covered by the mark tabsize columns. */
+void do_unindent_marked_void(void)
+{
+    do_indent_marked(-tabsize);
+}
+#endif /* !NANO_TINY */
+
 /* Someone hits Enter *gasp!* */
 void do_enter(void)
 {
@@ -293,7 +460,7 @@ bool execute_command(const char *command)
 	exit(0);
     }
 
-    /* Else continue as parent. */
+    /* Continue as parent. */
     close(fd[1]);
 
     if (pid == -1) {
@@ -349,7 +516,7 @@ bool execute_command(const char *command)
 #endif /* !NANO_TINY */
 
 #ifndef DISABLE_WRAPPING
-/* Clear the prepend_wrap flag.  We need to do this as soon as we do
+/* Unset the prepend_wrap flag.  We need to do this as soon as we do
  * something other than type text. */
 void wrap_reset(void)
 {
@@ -357,8 +524,8 @@ void wrap_reset(void)
 }
 
 /* We wrap the given line.  Precondition: we assume the cursor has been
- * moved forward since the last typed character.  Return value: whether
- * we wrapped. */
+ * moved forward since the last typed character.  Return TRUE if we
+ * wrapped, and FALSE otherwise. */
 bool do_wrap(filestruct *line)
 {
     size_t line_len;
@@ -392,8 +559,8 @@ bool do_wrap(filestruct *line)
     /* Step 1, finding where to wrap.  We are going to add a new line
      * after a blank character.  In this step, we call break_line() to
      * get the location of the last blank we can break the line at, and
-     * and set wrap_loc to the location of the character after it, so
-     * that the blank is preserved at the end of the line.
+     * set wrap_loc to the location of the character after it, so that
+     * the blank is preserved at the end of the line.
      *
      * If there is no legal wrap point, or we reach the last character
      * of the line while trying to find one, we should return without
@@ -439,9 +606,8 @@ bool do_wrap(filestruct *line)
     /* Step 2, making the new wrap line.  It will consist of indentation
      * followed by the text after the wrap point, optionally followed by
      * a space (if the text after the wrap point doesn't end in a blank)
-     * and the text of the next line, if they can fit without
-     * wrapping, the next line exists, and the prepend_wrap flag is
-     * set. */
+     * and the text of the next line, if they can fit without wrapping,
+     * the next line exists, and the prepend_wrap flag is set. */
 
     /* after_break is the text that will be wrapped to the next line. */
     after_break = line->data + wrap_loc;
@@ -553,8 +719,8 @@ bool do_wrap(filestruct *line)
      * be prepended to the next line. */
     prepend_wrap = TRUE;
 
-    /* Each line knows its line number.  We recalculate these if we
-     * inserted a new line. */
+    /* Each line knows its number.  We recalculate these if we inserted
+     * a new line. */
     if (!prepending)
 	renumber(line);
 
@@ -607,14 +773,14 @@ ssize_t break_line(const char *line, ssize_t goal
 	 * found with short enough display width.  */
     ssize_t cur_loc = 0;
 	/* Current index in line. */
+    size_t cur_pos = 0;
+	/* Current column position in line. */
     int line_len;
 
     assert(line != NULL);
 
-    while (*line != '\0' && goal >= 0) {
-	size_t pos = 0;
-
-	line_len = parse_mbchar(line, NULL, &pos);
+    while (*line != '\0' && goal >= cur_pos) {
+	line_len = parse_mbchar(line, NULL, &cur_pos);
 
 	if (is_blank_mbchar(line)
 #ifndef DISABLE_HELP
@@ -629,12 +795,11 @@ ssize_t break_line(const char *line, ssize_t goal
 #endif
 	}
 
-	goal -= pos;
 	line += line_len;
 	cur_loc += line_len;
     }
 
-    if (goal >= 0)
+    if (goal >= cur_pos)
 	/* In fact, the whole line displays shorter than goal. */
 	return cur_loc;
 
@@ -675,6 +840,11 @@ ssize_t break_line(const char *line, ssize_t goal
 	|| (newline && *line == '\n')
 #endif
 	)) {
+#ifndef DISABLE_HELP
+	if (newline && *line == '\n')
+	    break;
+#endif
+
 	line_len = parse_mbchar(line, NULL, NULL);
 
 	line += line_len;
@@ -1076,7 +1246,7 @@ void backup_lines(filestruct *first_line, size_t par_len)
 /* Find the beginning of the current paragraph if we're in one, or the
  * beginning of the next paragraph if we're not.  Afterwards, save the
  * quote length and paragraph length in *quote and *par.  Return TRUE if
- * we found a paragraph, or FALSE if there was an error or we didn't
+ * we found a paragraph, and FALSE if there was an error or we didn't
  * find a paragraph.
  *
  * See the comment at begpar() for more about when a line is the
@@ -1276,7 +1446,7 @@ void do_justify(bool full_justify)
 	indent_string = mallocstrcpy(NULL, "");
 
 	/* Find the first indentation in the paragraph that doesn't
-	 * match the indentation of the first line, and save itin
+	 * match the indentation of the first line, and save it in
 	 * indent_string.  If all the indentations are the same, save
 	 * the indentation of the first line in indent_string. */
 	{
@@ -1483,6 +1653,15 @@ void do_justify(bool full_justify)
 
     edit_refresh();
 
+#ifndef NANO_TINY
+    /* We're going to set jump_buf so that we return here after a
+     * SIGWINCH instead of to main().  Indicate this. */
+    jump_buf_main = FALSE;
+
+    /* Return here after a SIGWINCH. */
+    sigsetjmp(jump_buf, 1);
+#endif
+
     statusbar(_("Can now UnJustify!"));
 
     /* If constant cursor position display is on, make sure the current
@@ -1499,8 +1678,7 @@ void do_justify(bool full_justify)
     kbinput = do_input(&meta_key, &func_key, &s_or_t, &ran_func,
 	&finished, FALSE);
 
-    if (!meta_key && !func_key && s_or_t &&
-	kbinput == NANO_UNJUSTIFY_KEY) {
+    if (s_or_t && kbinput == NANO_UNJUSTIFY_KEY) {
 	/* Splice the justify buffer back into the file, but only if we
 	 * actually justified something. */
 	if (first_par_line != NULL) {
@@ -2133,7 +2311,7 @@ void do_spell(void)
     const char *spell_msg;
 
     if (temp == NULL) {
-	statusbar(_("Could not create temp file: %s"), strerror(errno));
+	statusbar(_("Error writing temp file: %s"), strerror(errno));
 	return;
     }
 
@@ -2155,6 +2333,8 @@ void do_spell(void)
     unlink(temp);
     free(temp);
 
+    currshortcut = main_list;
+
     /* If the spell-checker printed any error messages onscreen, make
      * sure that they're cleared off. */
     total_refresh();
@@ -2245,16 +2425,23 @@ void do_verbatim_input(void)
     size_t kbinput_len, i;
     char *output;
 
+    /* TRANSLATORS: This is displayed when the next keystroke will be
+     * inserted verbatim. */
     statusbar(_("Verbatim Input"));
 
-    /* If constant cursor position display is on, make sure the current
-     * cursor position will be properly displayed on the statusbar. */
-    if (ISSET(CONST_UPDATE))
-	do_cursorpos(TRUE);
-
     /* Read in all the verbatim characters. */
     kbinput = get_verbatim_kbinput(edit, &kbinput_len);
 
+    /* If constant cursor position display is on, make sure the current
+     * cursor position will be properly displayed on the statusbar.
+     * Otherwise, blank the statusbar. */
+    if (ISSET(CONST_UPDATE))
+	do_cursorpos(TRUE);
+    else {
+	blank_statusbar();
+	wnoutrefresh(bottomwin);
+    }
+
     /* Display all the verbatim characters at once, not filtering out
      * control characters. */
     output = charalloc(kbinput_len + 1);
