@@ -1,4 +1,4 @@
-/* $Id: utils.c,v 1.112 2006/01/09 03:23:29 dolorous Exp $ */
+/* $Id: utils.c,v 1.118 2006/06/21 16:54:51 dolorous Exp $ */
 /**************************************************************************
  *   utils.c                                                              *
  *                                                                        *
@@ -33,12 +33,10 @@
 /* Return the number of decimal digits in n. */
 int digits(size_t n)
 {
-    int i = 1;
+    int i;
 
-    while (n > 10) {
-	n /= 10;
-	i++;
-    }
+    for (i = 1; n >= 10; n /= 10, i++)
+	;
 
     return i;
 }
@@ -197,7 +195,8 @@ ssize_t ngetdelim(char **lineptr, size_t *n, int delim, FILE *stream)
     int c;
 
     /* Sanity checks. */
-    if (lineptr == NULL || n == NULL || stream == NULL) {
+    if (lineptr == NULL || n == NULL || stream == NULL ||
+	fileno(stream) == -1) {
 	errno = EINVAL;
 	return -1;
     }
@@ -241,18 +240,6 @@ ssize_t ngetdelim(char **lineptr, size_t *n, int delim, FILE *stream)
 #endif /* !NANO_TINY && ENABLE_NANORC */
 
 #ifdef HAVE_REGEX_H
-#ifdef BROKEN_REGEXEC
-/* Work around a potential segfault in glibc 2.2.3's regexec(). */
-int safe_regexec(const regex_t *preg, const char *string, size_t nmatch,
-	regmatch_t pmatch[], int eflags)
-{
-    if (string != NULL && *string != '\0')
-	return regexec(preg, string, nmatch, pmatch, eflags);
-
-    return REG_NOMATCH;
-}
-#endif
-
 /* Do the compiled regex in preg and the regex in string match the
  * beginning or end of a line? */
 bool regexp_bol_or_eol(const regex_t *preg, const char *string)
@@ -261,7 +248,7 @@ bool regexp_bol_or_eol(const regex_t *preg, const char *string)
 	regexec(preg, string, 0, NULL, REG_NOTBOL | REG_NOTEOL) ==
 	REG_NOMATCH);
 }
-#endif /* HAVE_REGEX_H */
+#endif
 
 #ifndef DISABLE_SPELLER
 /* Is the word starting at position pos in buf a whole word? */
@@ -354,17 +341,15 @@ const char *strstrwrapper(const char *haystack, const char *needle,
     return mbstrcasestr(start, needle);
 }
 
-/* This is a wrapper for the perror() function.  The wrapper takes care
- * of curses, calls perror() (which writes to stderr), and then
- * refreshes the screen.  Note that nperror() causes the window to
- * flicker once. */
+/* This is a wrapper for the perror() function.  The wrapper temporarily
+ * leaves curses mode, calls perror() (which writes to stderr), and then
+ * reenters curses mode, updating the screen in the process.  Note that
+ * nperror() causes the window to flicker once. */
 void nperror(const char *s)
 {
-    /* Leave curses mode and go to the terminal. */
-    if (endwin() != ERR) {
-	perror(s);		/* Print the error. */
-	total_refresh();	/* Return to curses and refresh. */
-    }
+    endwin();
+    perror(s);
+    doupdate();
 }
 
 /* This is a wrapper for the malloc() function that properly handles
@@ -596,22 +581,6 @@ size_t get_totsize(const filestruct *begin, const filestruct *end)
     return totsize;
 }
 
-#ifndef NDEBUG
-/* Return what the current line number should be, starting at edittop
- * and ending at fileptr. */
-int check_linenumbers(const filestruct *fileptr)
-{
-    int check_line = 0;
-    const filestruct *filetmp;
-
-    for (filetmp = openfile->edittop; filetmp != fileptr;
-	filetmp = filetmp->next)
-	check_line++;
-
-    return check_line;
-}
-#endif /* !NDEBUG */
-
 #ifdef DEBUG
 /* Dump the filestruct inptr to stderr. */
 void dump_filestruct(const filestruct *inptr)
